#!/usr/bin/env python3

"""
The clean main function.
"""

import argparse
import json
import os
import sys
from typing import cast

import requests
import yaml

import c2cciutils


def clean(image: str, tag: str, token: str) -> None:
    """
    Delete an image from Docker hub.

    Arguments:
        image: The image name that should be deleted (<organization>/<name>)
        tag: The tag that should be deleted
        token: The token used to be authenticated on Docker hub
    """

    print(f"Delete image '{image}:{tag}'.")

    response = requests.head(
        f"https://hub.docker.com/v2/repositories/{image}/tags/{tag}/",
        headers={"Authorization": "JWT " + token},
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    )
    if response.status_code == 404:
        return
    if not response.ok:
        print(f"Error checking image '{image}:{tag}' status.")
        print(response.text)
        sys.exit(1)

    response = requests.delete(
        f"https://hub.docker.com/v2/repositories/{image}/tags/{tag}/",
        headers={"Authorization": "JWT " + token},
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    )
    if not response.ok:
        print("::error::Error on deleting tag: " + tag)
        print(response.text)
        sys.exit(1)


def main() -> None:
    """
    Run the main function.
    """
    parser = argparse.ArgumentParser(
        description=(
            "Clean the Docker images on Docker Hub for the branch we delete "
            "(get from the GitHub event information)."
        )
    )
    parser.parse_args()

    username = (
        os.environ["DOCKERHUB_USERNAME"]
        if "DOCKERHUB_USERNAME" in os.environ
        else c2cciutils.gopass("gs/ci/dockerhub/username")
    )
    password = (
        os.environ["DOCKERHUB_PASSWORD"]
        if "DOCKERHUB_PASSWORD" in os.environ
        else c2cciutils.gopass("gs/ci/dockerhub/password")
    )
    token = requests.post(
        "https://hub.docker.com/v2/users/login/",
        headers={"Content-Type": "application/json"},
        data=json.dumps(
            {
                "username": username,
                "password": password,
            }
        ),
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    ).json()["token"]

    with open(os.environ["GITHUB_EVENT_PATH"], encoding="utf-8") as event_file:
        event = json.loads(event_file.read())
        print(yaml.dump(event))
        if "pull_request" in event:
            ref = str(event["number"])
        else:
            ref = event["ref"]

    ref = ref.replace("/", "_")

    config = c2cciutils.get_config()

    docker_config = cast(
        c2cciutils.configuration.PublishDockerConfig,
        config.get("publish", {}).get("docker", {}) if config.get("publish", {}).get("docker", False) else {},
    )
    for image in docker_config.get("images", []):
        for tag in image.get("tags", []):
            clean(image["name"], tag.format(version=ref), token)


if __name__ == "__main__":
    main()



"""
The auditing functions.
"""

import datetime
import json
import os.path
import subprocess  # nosec
import sys
from argparse import Namespace

import c2cciutils
import c2cciutils.configuration
import c2cciutils.security


def print_versions(
    config: c2cciutils.configuration.PrintVersions,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Print the versions.

    Arguments:
        config: The audit section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del full_config, args

    print("::group::Versions")
    c2cciutils.print_versions(config)
    print("::endgroup::")

    return True


def snyk(
    config: c2cciutils.configuration.AuditSnykConfig,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Audit the code with Snyk.
    """
    del full_config

    one_done = False
    install_success = True
    test_success = True

    for file in (
        subprocess.run(
            ["git", "ls-files", "requirements.txt", "*/requirements.txt"], stdout=subprocess.PIPE, check=True
        )
        .stdout.decode()
        .strip()
        .split("\n")
    ):
        if not file:
            continue
        if file in config.get(
            "files_no_install", c2cciutils.configuration.AUDIT_SNYK_FILES_NO_INSTALL_DEFAULT
        ):
            continue
        print(f"::notice::Install from: {file}")
        if not one_done:
            print("::group::Install dependencies")
            one_done = True
        sys.stdout.flush()
        sys.stderr.flush()
        proc = subprocess.run(  # pylint: disable=subprocess-run-check
            [
                "pip",
                "install",
                *config.get(
                    "pip_install_arguments", c2cciutils.configuration.AUDIT_SNYK_PIP_INSTALL_ARGUMENTS_DEFAULT
                ),
                f"--requirement={file}",
            ]
        )
        if proc.returncode != 0:
            print(f"::error::With error from: {file}")
        install_success &= proc.returncode == 0

    for file in (
        subprocess.run(["git", "ls-files", "Pipfile", "*/Pipfile"], stdout=subprocess.PIPE, check=True)
        .stdout.decode()
        .strip()
        .split("\n")
    ):
        if not file:
            continue
        if file in config.get(
            "files_no_install", c2cciutils.configuration.AUDIT_SNYK_FILES_NO_INSTALL_DEFAULT
        ):
            continue
        if not one_done:
            print("::group::Install dependencies")
            one_done = True
        print(f"::notice::Install from: {file}")
        directory = os.path.dirname(os.path.abspath(file))

        sys.stdout.flush()
        sys.stderr.flush()
        proc = subprocess.run(  # pylint: disable=subprocess-run-check
            [
                "pipenv",
                "sync",
                *config.get(
                    "pipenv_sync_arguments", c2cciutils.configuration.AUDIT_SNYK_PIPENV_SYNC_ARGUMENTS_DEFAULT
                ),
            ],
            cwd=directory,
        )
        if proc.returncode != 0:
            print(f"::error::With error from: {file}")
        install_success &= proc.returncode == 0

    if one_done:
        print("::endgroup::")
    if not install_success:
        print("::error::Error while installing the dependencies")

    snyk_exec, env = c2cciutils.snyk_exec()
    if not args.fix:
        command = [snyk_exec, "monitor", f"--target-reference={args.branch}"] + config.get(
            "monitor_arguments", c2cciutils.configuration.AUDIT_SNYK_MONITOR_ARGUMENTS_DEFAULT
        )
        print(f"::group::Run: {' '.join(command)}")
        sys.stdout.flush()
        sys.stderr.flush()
        subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
        print("::endgroup::")

        command = [snyk_exec, "test"] + config.get(
            "test_arguments", c2cciutils.configuration.AUDIT_SNYK_TEST_ARGUMENTS_DEFAULT
        )
        print(f"::group::Run: {' '.join(command)}")
        sys.stdout.flush()
        sys.stderr.flush()
        test_proc = subprocess.run(command, env=env)  # pylint: disable=subprocess-run-check
        print("::endgroup::")
        if test_proc.returncode != 0:
            test_success = False
            print("::error::With error")

        # Clean all the changes to isolate the fix diff
        subprocess.run(["git", "reset", "--hard"], check=True)

    command = [snyk_exec, "fix"] + config.get(
        "fix_arguments", c2cciutils.configuration.AUDIT_SNYK_FIX_ARGUMENTS_DEFAULT
    )
    print(f"::group::Run: {' '.join(command)}")
    sys.stdout.flush()
    sys.stderr.flush()
    snyk_fix_proc = subprocess.run(  # pylint: disable=subprocess-run-check
        command, env={**env, "FORCE_COLOR": "false"}, stdout=subprocess.PIPE, encoding="utf-8"
    )
    snyk_fix_message = snyk_fix_proc.stdout.strip()
    print("::endgroup::")

    if not args.fix:
        diff_proc = subprocess.run(["git", "diff", "--quiet"])  # pylint: disable=subprocess-run-check
        if diff_proc.returncode != 0:
            print("::error::There is some changes to commit")
            print("::group::Diff")
            sys.stdout.flush()
            sys.stderr.flush()
            subprocess.run(["git", "diff"], check=True)
            print("::endgroup::")

            current_branch = c2cciutils.get_branch(args.branch)
            subprocess.run(["git", "checkout", "-b", f"snyk-fix/{current_branch}"], check=True)
            subprocess.run(["git", "add", "--all"], check=True)
            subprocess.run(["git", "commit", "--message=Snyk auto fix"], check=True)
            if os.environ.get("TEST") != "TRUE":
                subprocess.run(
                    ["git", "push", "--force", "origin", f"snyk-fix/{current_branch}"],
                    check=True,
                )
                env = os.environ.copy()
                if "GH_TOKEN" not in env:
                    if "GITHUB_TOKEN" in env:
                        env["GH_TOKEN"] = env["GITHUB_TOKEN"]
                    else:
                        env["GH_TOKEN"] = str(c2cciutils.gopass("gs/ci/github/token/gopass"))
                fix_github_create_pull_request_arguments = config.get(
                    "fix_github_create_pull_request_arguments",
                    c2cciutils.configuration.AUDIT_SNYK_FIX_PULL_REQUEST_ARGUMENTS_DEFAULT,
                )
                subprocess.run(
                    [
                        "gh",
                        "pr",
                        "create",
                        f"--base={current_branch}",
                        f"--body={snyk_fix_message}",
                        *fix_github_create_pull_request_arguments,
                    ],
                    check=True,
                    env=env,
                )
            subprocess.run(["git", "checkout", current_branch], check=True)

    return install_success and test_success and diff_proc.returncode == 0


def outdated_versions(
    config: None,
    full_config: c2cciutils.configuration.Configuration,
    args: Namespace,
) -> bool:
    """
    Check that the versions from the SECURITY.md are not outdated.

    Arguments:
        config: The audit section config
        full_config: All the CI config
        args: The parsed command arguments
    """
    del config, full_config

    repo = c2cciutils.get_repository().split("/")
    json_response = c2cciutils.graphql(
        "default_branch.graphql",
        {"name": repo[1], "owner": repo[0]},
    )

    if "errors" in json_response:
        raise RuntimeError(json.dumps(json_response["errors"], indent=2))
    if json_response["repository"]["defaultBranchRef"]["name"] != c2cciutils.get_branch(args.branch):
        return True

    success = True

    if not os.path.exists("SECURITY.md"):
        return True

    with open("SECURITY.md", encoding="utf-8") as security_file:
        security = c2cciutils.security.Security(security_file.read())

    version_index = security.headers.index("Version")
    date_index = security.headers.index("Supported Until")

    for row in security.data:
        str_date = row[date_index]
        if str_date not in ("Unsupported", "Best effort", "To be defined"):
            date = datetime.datetime.strptime(row[date_index], "%d/%m/%Y")
            if date < datetime.datetime.now():
                c2cciutils.error(
                    "versions",
                    f"The version '{row[version_index]}' is outdated, it can be set to "
                    "'Unsupported', 'Best effort' or 'To be defined'",
                    "SECURITY.md",
                )
                success = False
    return success



#!/usr/bin/env python3

import argparse
import subprocess  # nosec
import sys


def _print(message: str) -> None:
    print(message)
    sys.stdout.flush()


def main() -> None:
    """Get some logs to from k8s."""
    parser = argparse.ArgumentParser(description="Get some logs to from k8s.")
    parser.add_argument("--namespace", help="Namespace to be used")

    args = parser.parse_args()

    if args.namespace:
        subprocess.run(["kubectl", "config", "set-context", "--current", "--namespace=default"], check=True)

    try:
        _print("::group::Events")
        subprocess.run(["kubectl", "get", "events"], check=False)
        _print("::endgroup::")

        _print("::group::Deployments")
        subprocess.run(["kubectl", "get", "deployments", "--output=wide"], check=False)
        _print("::endgroup::")

        _print("::group::Pods")
        subprocess.run(["kubectl", "get", "pods", "--output=wide"], check=False)
        _print("::endgroup::")

        for name in (
            subprocess.run(["kubectl", "get", "pods", "--output=name"], check=True, stdout=subprocess.PIPE)
            .stdout.decode()
            .split("\n")
        ):
            if name:
                _print(f"::group::{name}: Describe")
                subprocess.run(["kubectl", "describe", name], check=False)
                _print("::endgroup::")

                for container in (
                    subprocess.run(
                        ["kubectl", "get", name, "--output=jsonpath={.spec.initContainers[*].name}"],
                        check=True,
                        stdout=subprocess.PIPE,
                    )
                    .stdout.decode()
                    .split()
                ):
                    if name:
                        _print(f"::group::{name} {container}: Logs")
                        subprocess.run(["kubectl", "logs", name, container], check=False)
                        _print("::endgroup::")

                for container in (
                    subprocess.run(
                        ["kubectl", "get", name, "--output=jsonpath={.spec.containers[*].name}"],
                        check=True,
                        stdout=subprocess.PIPE,
                    )
                    .stdout.decode()
                    .split()
                ):
                    _print(f"::group::{name} {container}: Logs")
                    subprocess.run(["kubectl", "logs", name, container], check=False)
                    _print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        # No exit error
        print(exception)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import argparse
import subprocess  # nosec
import sys

import c2cciutils
import c2cciutils.configuration
import c2cciutils.scripts.download_applications


def _print(message: str) -> None:
    print(message)
    sys.stdout.flush()


def main() -> None:
    """Get some logs to from k8s."""
    parser = argparse.ArgumentParser(description="Install k3d/k3s and create a cluster.")
    _ = parser.parse_args()

    config = c2cciutils.get_config()

    _print("::group::Install")
    c2cciutils.scripts.download_applications.download_c2cciutils_applications()
    _print("::endgroup::")

    _print("::group::Create cluster")
    for cmd in (
        config.get("k8s", {})
        .get("k3d", {})
        .get("install-commands", c2cciutils.configuration.K3D_INSTALL_COMMANDS_DEFAULT)
    ):
        subprocess.run(cmd, check=True)
    _print("::endgroup::")


if __name__ == "__main__":
    main()



"""
Read the table of versions from SECURITY.md.
"""

import xml.etree.ElementTree  # nosec
from typing import List, Optional

import markdown
from markdown.extensions.tables import TableExtension


class Security:
    """
    Read the table of versions from SECURITY.md.
    """

    headers: List[str]
    data: List[List[str]]
    _row: Optional[List[str]] = None

    def __init__(self, status: str):
        """
        Initialize.

        Arguments:
            status: the content of the SECURITY.md file.
        """

        self.headers = []
        self.data = []

        markdown_instance = markdown.Markdown(extensions=[TableExtension()])

        elem = markdown_instance.parser.parseDocument(
            [s for s in status.split("\n") if s != "" and s[0] != "#" and s[0] != "["]
        )
        self._pe(elem.getroot())

        self.data = [r for r in self.data if len([c for c in r if c is not None]) > 0]
        for row in self.data:
            row.append("")

    def _pe(self, elem: xml.etree.ElementTree.Element) -> None:
        """
        Parse the HTML table.

        Arguments:
            elem: The XML element
        """
        if elem.tag == "th":
            assert elem.text is not None
            self.headers.append(elem.text)
        if elem.tag == "tr":
            self._row = []
            self.data.append(self._row)
        if elem.tag == "td":
            self._row.append(elem.text)  # type: ignore
        for element in list(elem):
            self._pe(element)



#!/usr/bin/env python3

import argparse
import json
import subprocess  # nosec
import sys
import time
from typing import Any


def _check_deployment_status(deployments: Any) -> bool:
    for deployment in deployments["items"]:
        if not deployment["status"]:
            print(f'Waiting status for {deployment["metadata"]["name"]}')
            return False

        for condition in deployment["status"].get("conditions", []):
            if not condition["status"]:
                print(
                    f'::group::Deployment {deployment["metadata"]["name"]} not ready: {condition["message"]}'
                )
                print(json.dumps(condition, indent=4))
                print("::endgroup::")
                return False

        if deployment["status"].get("unavailableReplicas", 0) != 0:
            print(
                f'::group::Deployment {deployment["metadata"]["name"]} not ready there is {deployment["status"].get("unavailableReplicas", 0)} '
                "unavailable replicas"
            )
            print(json.dumps(deployment["status"], indent=4))
            print("::endgroup::")
            return False

    return True


def _check_container_status(pod: Any, status: Any, is_init: bool = False) -> bool:
    del is_init
    good = status["ready"]

    if not good:
        waiting = status["state"].get("waiting")
        terminated = status["state"].get("terminated")
        state = waiting if waiting else terminated
        status_message = state.get("message", state.get("reason", "")) if state else ""
        if not status_message:
            state = status.get("lastState", {}).get("terminated", {})
            status_message = state.get("message", state.get("reason", "")) if state else ""
        status_message_long = status_message.strip()
        if "message" in status.get("lastState", {}).get("terminated", {}):
            status_message_long = status["lastState"]["terminated"]["message"]
        status_message = status_message.split("\n")[0]
        status_message = status_message.strip()
        if status_message == "Completed":
            return True
        print(f'::group::Container not ready in {pod["metadata"]["name"]}: {status_message}')
        if status_message_long != status_message:
            print(status_message_long)
        print(json.dumps(status, indent=4))
        print("::endgroup::")
        return False
    return True


def _check_pod_status(pods: Any) -> bool:
    for pod in pods["items"]:
        for condition in pod["status"].get("conditions", []):
            if not condition["status"]:
                print(
                    f'::group::Pod not ready in {pod["metadata"]["name"]}: {condition.get("message", condition["type"])}'
                )
                print(json.dumps(condition, indent=4))
                print("::endgroup::")
                return False

        for status in pod["status"].get("initContainerStatuses", []):
            if not _check_container_status(pod, status, True):
                return False
        for status in pod["status"].get("containerStatuses", []):
            if not _check_container_status(pod, status):
                return False

        if pod["status"].get("phase") not in ("Running", "Succeeded"):
            print(f'::group::The Pod {pod["metadata"]["name"]} is not ready: {pod["status"].get("phase")}')
            return False

    return True


def main() -> None:
    """Wait that the k8s application is ready."""
    parser = argparse.ArgumentParser(description="Get some logs to from k8s.")
    parser.add_argument("--namespace", help="Namespace to be used")
    parser.add_argument(
        "-l",
        "--selector",
        default="",
        help="Selector (label query) to filter on, supports '=', '==', and '!='.(e.g. -l key1=value1,key2=value2)",
    )
    parser.add_argument("--no-deployments", dest="deployments", action="store_false")
    parser.add_argument(
        "--nb-try", default=20, type=int, help="Number of try to wait for the application to be ready"
    )
    parser.add_argument("--sleep", default=10, type=int, help="Sleep time before each try")

    args = parser.parse_args()

    if args.namespace:
        subprocess.run(["kubectl", "config", "set-context", "--current", "--namespace=default"], check=True)

    for _ in range(args.nb_try):
        time.sleep(args.sleep)
        success = True
        deployements_names = []
        if args.deployments:
            deployments = subprocess.run(
                ["kubectl", "get", "deployments", "--output=json"],
                stdout=subprocess.PIPE,
                check=True,
            )
            deployements_json = json.loads(deployments.stdout)
            success &= _check_deployment_status(deployements_json)
            deployements_names = [
                deployment["metadata"]["name"] for deployment in json.loads(deployments.stdout)["items"]
            ]

        pods = subprocess.run(
            ["kubectl", "get", "pods", "--output=json", f"--selector={args.selector}"],
            stdout=subprocess.PIPE,
            check=True,
        )
        pods_json = json.loads(pods.stdout)
        success &= _check_pod_status(pods_json)
        pods_name = [p["metadata"]["name"] for p in json.loads(pods.stdout)["items"]]
        if success:
            if deployements_names:
                print()
                print("Deployments ready:")
                print("\n".join(deployements_names))
            print()
            print("Pods ready:")
            print("\n".join(pods_name))
            sys.exit(0)
    sys.exit(1)


if __name__ == "__main__":
    main()



import pkg_resources
import utils
from package import Package


class ReqPackage(Package):
    """Wrapper class for Requirements instance
      :param obj: The `Requirements` instance to wrap over
      :param dist: optional `pkg_resources.Distribution` instance for
                   this requirement
    """

    UNKNOWN_VERSION = '?'

    def __init__(self, obj, dist=None):
        super(ReqPackage, self).__init__(obj)
        self.dist = dist

    @property
    def version_spec(self):
        specs = self._obj.specs
        return ','.join([''.join(sp) for sp in specs]) if specs else None

    @property
    def installed_version(self):
        if not self.dist:
            return utils.guess_version(self.key, self.UNKNOWN_VERSION)
        return self.dist.version

    def is_conflicting(self):
        """If installed version conflicts with required version"""
        # unknown installed version is also considered conflicting
        if self.installed_version == self.UNKNOWN_VERSION:
            return True
        ver_spec = (self.version_spec if self.version_spec else '')
        req_version_str = '{0}{1}'.format(self.project_name, ver_spec)
        req_obj = pkg_resources.Requirement.parse(req_version_str)
        return self.installed_version not in req_obj

    def as_dict(self):
        return {'key': self.key,
                'package_name': self.project_name,
                'installed_version': self.installed_version,
                'required_version': self.version_spec}



from importlib import import_module
from operator import attrgetter
try:
    from collections import OrderedDict
except ImportError:
    from ordereddict import OrderedDict
import sys
from reqPackage import ReqPackage
from distPackage import DistPackage
__version__ = '0.10.1'


def build_dist_index(pkgs):
    """Build an index pkgs by their key as a dict.
    :param list pkgs: list of pkg_resources.Distribution instances
    :returns: index of the pkgs by the pkg key
    :rtype: dict
    """
    return dict((p.key, DistPackage(p)) for p in pkgs)


def construct_tree(index):
    """Construct tree representation of the pkgs from the index.
    The keys of the dict representing the tree will be objects of type
    DistPackage and the values will be list of ReqPackage objects.
    :param dict index: dist index ie. index of pkgs by their keys
    :returns: tree of pkgs and their dependencies
    :rtype: dict
    """
    return dict((p, [ReqPackage(r, index.get(r.key))
                     for r in p.requires()])
                for p in index.values())


def sorted_tree(tree):
    """Sorts the dict representation of the tree
    The root packages as well as the intermediate packages are sorted
    in the alphabetical order of the package names.
    :param dict tree: the pkg dependency tree obtained by calling
                     `construct_tree` function
    :returns: sorted tree
    :rtype: collections.OrderedDict
    """
    return OrderedDict(sorted([(k, sorted(v, key=attrgetter('key')))
                               for k, v in tree.items()],
                              key=lambda kv: kv[0].key))


def guess_version(pkg_key, default='?'):
    """Guess the version of a pkg when pip doesn't provide it
    :param str pkg_key: key of the package
    :param str default: default version to return if unable to find
    :returns: version
    :rtype: string
    """
    try:
        m = import_module(pkg_key)
    except ImportError:
        return default
    else:
        return getattr(m, '__version__', default)


def is_string(obj):
    """Check whether an object is a string"""
    if sys.version_info < (3,):
        # Python 2.x only
        return isinstance(obj, basestring)
    else:
        return isinstance(obj, str)



#!/usr/bin/env python3

"""
The publish script.
"""

import argparse
import os
import re
import subprocess  # nosec
import sys
import tarfile
from typing import Dict, List, Match, Optional, Set, cast

import requests
import yaml

import c2cciutils
import c2cciutils.configuration
import c2cciutils.lib.docker
import c2cciutils.publish
import c2cciutils.security
from c2cciutils.publish import GoogleCalendar
from c2cciutils.scripts.trigger_image_update import dispatch


def match(tpe: str, base_re: str) -> Optional[Match[str]]:
    """
    Return the match for `GITHUB_REF` basically like: `refs/<tpe>/<base_re>`.

    Arguments:
        tpe: The type of ref we want to match (heads, tag, ...)
        base_re: The regular expression to match the value
    """
    if base_re[0] == "^":
        base_re = base_re[1:]
    if base_re[-1] != "$":
        base_re += "$"
    return re.match(f"^refs/{tpe}/{base_re}", os.environ["GITHUB_REF"])


def to_version(full_config: c2cciutils.configuration.Configuration, value: str, kind: str) -> str:
    """
    Compute publish version from branch name or tag.

    Arguments:
        full_config: The full configuration
        value: The value to be transformed
        kind: The name of the transformer in the configuration
    """
    item_re = c2cciutils.compile_re(
        cast(
            c2cciutils.configuration.VersionTransform, full_config["version"].get(kind + "_to_version_re", [])
        )
    )
    value_match = c2cciutils.match(value, item_re)
    if value_match[0] is not None:
        return c2cciutils.get_value(*value_match)
    return value


def main() -> None:
    """
    Run the publish.
    """
    parser = argparse.ArgumentParser(description="Publish the project.")
    parser.add_argument("--group", default="default", help="The publishing group")
    parser.add_argument("--version", help="The version to publish to")
    parser.add_argument(
        "--docker-versions",
        help="The versions to publish on Docker registry, comma separated, ex: 'x,x.y,x.y.z,latest'.",
    )
    parser.add_argument("--snyk-version", help="The version to publish to Snyk")
    parser.add_argument("--branch", help="The branch from which to compute the version")
    parser.add_argument("--tag", help="The tag from which to compute the version")
    parser.add_argument("--dry-run", action="store_true", help="Don't do the publish")
    parser.add_argument(
        "--type",
        help="The type of version, if no argument provided auto-determinate, can be: "
        "rebuild (in case of rebuild), version_tag, version_branch, feature_branch, feature_tag "
        "(for pull request)",
    )
    args = parser.parse_args()

    config = c2cciutils.get_config()

    if config["publish"].get("print_versions"):
        print("::group::Versions")
        c2cciutils.print_versions(config.get("publish", {}).get("print_versions", {}))
        print("::endgroup::")

    # Describe the kind of release we do: rebuild (specified with --type), version_tag, version_branch,
    # feature_branch, feature_tag (for pull request)
    version: str = ""
    ref = os.environ.get("GITHUB_REF", "refs/heads/fake-local")

    if len([e for e in [args.version, args.branch, args.tag] if e is not None]) > 1:
        print("::error::you specified more than one of the arguments --version, --branch or --tag")
        sys.exit(1)

    version_type = args.type

    tag_match = c2cciutils.match(
        ref,
        c2cciutils.compile_re(config["version"].get("tag_to_version_re", []), "refs/tags/"),
    )
    branch_match = c2cciutils.match(
        ref,
        c2cciutils.compile_re(config["version"].get("branch_to_version_re", []), "refs/heads/"),
    )
    ref_match = re.match(r"refs/pull/(.*)/merge", ref)

    if args.version is not None:
        version = args.version
    elif args.branch is not None:
        version = to_version(config, args.branch, "branch")
    elif args.tag is not None:
        version = to_version(config, args.tag, "tag")
    elif tag_match[0] is not None:
        if version_type is None:
            version_type = "version_tag"
        else:
            print("::warning::you specified the argument --type but not one of --version, --branch or --tag")
        version = c2cciutils.get_value(*tag_match)
    elif branch_match[0] is not None:
        if version_type is None:
            version_type = "version_branch"
        else:
            print("::warning::you specified the argument --type but not one of --version, --branch or --tag")
        version = c2cciutils.get_value(*branch_match)
    elif ref_match is not None:
        version = c2cciutils.get_value(ref_match, {}, ref)
        if version_type is None:
            version_type = "feature_branch"
    elif ref.startswith("refs/heads/"):
        if version_type is None:
            version_type = "feature_branch"
        else:
            print("::warning::you specified the argument --type but not one of --version, --branch or --tag")
        # By the way we replace '/' by '_' because it isn't supported by Docker
        version = "_".join(ref.split("/")[2:])
    elif ref.startswith("refs/tags/"):
        if version_type is None:
            version_type = "feature_tag"
        else:
            print("::warning::you specified the argument --type but not one of --version, --branch or --tag")
        # By the way we replace '/' by '_' because it isn't supported by Docker
        version = "_".join(ref.split("/")[2:])
    else:
        print(
            f"WARNING: {ref} is not supported, only ref starting with 'refs/heads/' or 'refs/tags/' "
            "are supported, ignoring"
        )
        sys.exit(0)

    if version_type is None:
        print(
            "::error::you specified one of the arguments --version, --branch or --tag but not the --type, GitHub ref is: {ref}"
        )
        sys.exit(1)

    if version_type is not None:
        if args.dry_run:
            print(f"Create release type {version_type}: {version} (dry run)")
        else:
            print(f"Create release type {version_type}: {version}")

    success = True
    pypi_config = cast(
        c2cciutils.configuration.PublishPypiConfig,
        config.get("publish", {}).get("pypi", {}) if config.get("publish", {}).get("pypi", False) else {},
    )
    if pypi_config:
        for package in pypi_config["packages"]:
            if package.get("group", c2cciutils.configuration.PUBLISH_PIP_PACKAGE_GROUP_DEFAULT) == args.group:
                publish = version_type in pypi_config.get("versions", [])
                if args.dry_run:
                    print(
                        f"{'Publishing' if publish else 'Checking'} "
                        f"'{package.get('path')}' to pypi, skipping (dry run)"
                    )
                else:
                    success &= c2cciutils.publish.pip(package, version, version_type, publish)

    google_calendar = None
    google_calendar_publish = config.get("publish", {}).get("google_calendar", False) is not False
    google_calendar_config = cast(
        c2cciutils.configuration.PublishGoogleCalendarConfig,
        config.get("publish", {}).get("google_calendar", {}),
    )

    docker_config = cast(
        c2cciutils.configuration.PublishDockerConfig,
        config.get("publish", {}).get("docker", {}) if config.get("publish", {}).get("docker", False) else {},
    )
    if docker_config:
        latest = False
        full_repo = c2cciutils.get_repository()
        full_repo_split = full_repo.split("/")
        master_branch, _ = c2cciutils.get_master_branch(full_repo_split)
        security_response = requests.get(
            f"https://raw.githubusercontent.com/{full_repo}/{master_branch}/SECURITY.md",
            headers=c2cciutils.add_authorization_header({}),
            timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
        )
        c2cciutils.check_response(security_response, False)
        if (
            security_response.ok
            and docker_config.get("latest", c2cciutils.configuration.PUBLISH_DOCKER_LATEST_DEFAULT) is True
        ):
            security = c2cciutils.security.Security(security_response.text)
            version_index = security.headers.index("Version")
            latest = security.data[-1][version_index] == version

        images_src: Set[str] = set()
        images_full: List[str] = []
        images_snyk: Set[str] = set()
        versions = args.docker_versions.split(",") if args.docker_versions else [version]
        for image_conf in docker_config.get("images", []):
            if (
                image_conf.get("group", c2cciutils.configuration.PUBLISH_DOCKER_IMAGE_GROUP_DEFAULT)
                == args.group
            ):
                for tag_config in image_conf.get(
                    "tags", c2cciutils.configuration.PUBLISH_DOCKER_IMAGE_TAGS_DEFAULT
                ):
                    tag_src = tag_config.format(version="latest")
                    image_source = f"{image_conf['name']}:{tag_src}"
                    images_src.add(image_source)
                    tag_snyk = tag_config.format(version=args.snyk_version or version)
                    image_snyk = f"{image_conf['name']}:{tag_snyk}"

                    # Workaround sine we have the business plan
                    image_snyk = f"{image_conf['name']}_{tag_snyk}"

                    if not args.dry_run:
                        subprocess.run(["docker", "tag", image_source, image_snyk], check=True)
                    images_snyk.add(image_snyk)
                    if tag_snyk != tag_src and not args.dry_run:
                        subprocess.run(
                            [
                                "docker",
                                "tag",
                                image_source,
                                f"{image_conf['name']}:{tag_snyk}",
                            ],
                            check=True,
                        )

                    tags_calendar = []
                    for name, conf in {
                        **cast(
                            Dict[str, c2cciutils.configuration.PublishDockerRepository],
                            c2cciutils.configuration.DOCKER_REPOSITORY_DEFAULT,
                        ),
                        **docker_config.get("repository", {}),
                    }.items():
                        for docker_version in versions:
                            tag_dst = tag_config.format(version=docker_version)
                            if tag_dst not in tags_calendar:
                                tags_calendar.append(tag_dst)
                            image_dst = f"{image_conf['name']}:{tag_dst}"
                            if version_type in conf.get(
                                "versions",
                                c2cciutils.configuration.PUBLISH_DOCKER_REPOSITORY_VERSIONS_DEFAULT,
                            ):
                                if args.dry_run:
                                    print(f"Publishing {image_dst} to {name}, skipping (dry run)")
                                    if latest:
                                        print(f"Publishing {image_source} to {name}, skipping (dry run)")
                                else:
                                    success &= c2cciutils.publish.docker(
                                        conf, name, image_conf, tag_src, tag_dst, latest, images_full
                                    )

                    if google_calendar_publish:
                        if version_type in google_calendar_config.get(
                            "on", c2cciutils.configuration.PUBLISH_GOOGLE_CALENDAR_ON_DEFAULT
                        ):
                            if not google_calendar:
                                google_calendar = GoogleCalendar()
                            summary = f"{image_conf['name']}:{', '.join(tags_calendar)}"
                            description = "\n".join(
                                [
                                    f"Published the image {image_conf['name']}",
                                    f"Published on: {', '.join(docker_config['repository'].keys())}",
                                    f"With tags: {', '.join(tags_calendar)}",
                                    f"For version type: {version_type}",
                                ]
                            )

                            google_calendar.create_event(summary, description)

        if args.dry_run:
            sys.exit(0)

        dispatch_config = docker_config.get("dispatch", {})
        if dispatch_config is not False and images_full:
            dispatch(
                dispatch_config.get(
                    "repository", c2cciutils.configuration.DOCKER_DISPATCH_REPOSITORY_DEFAULT
                ),
                dispatch_config.get(
                    "event-type", c2cciutils.configuration.DOCKER_DISPATCH_EVENT_TYPE_DEFAULT
                ),
                images_full,
            )

        snyk_exec, env = c2cciutils.snyk_exec()
        for image in images_snyk:
            print(f"::group::Snyk check {image}")
            sys.stdout.flush()
            sys.stderr.flush()
            try:
                if version_type in ("version_branch", "version_tag"):
                    monitor_args = docker_config.get("snyk", {}).get(
                        "monitor_args",
                        c2cciutils.configuration.PUBLISH_DOCKER_SNYK_MONITOR_ARGS_DEFAULT,
                    )
                    if monitor_args is not False:
                        subprocess.run(  # pylint: disable=subprocess-run-check
                            [
                                snyk_exec,
                                "container",
                                "monitor",
                                *monitor_args,
                                # Available only on the business plan
                                # f"--project-tags=tag={image.split(':')[-1]}",
                                image,
                            ],
                            env=env,
                        )
                test_args = docker_config.get("snyk", {}).get(
                    "test_args", c2cciutils.configuration.PUBLISH_DOCKER_SNYK_TEST_ARGS_DEFAULT
                )
                snyk_error = False
                if test_args is not False:
                    proc = subprocess.run(
                        [snyk_exec, "container", "test", *test_args, image],
                        check=False,
                        env=env,
                    )
                    if proc.returncode != 0:
                        snyk_error = True
                print("::endgroup::")
                if snyk_error:
                    print("::error::Critical vulnerability found by Snyk in the published image.")
            except subprocess.CalledProcessError as exception:
                print(f"Error: {exception}")
                print("::endgroup::")
                print("::error::With error")

        versions_config, dpkg_config_found = c2cciutils.lib.docker.get_versions_config()
        dpkg_success = True
        for image in images_src:
            dpkg_success &= c2cciutils.lib.docker.check_versions(versions_config.get(image, {}), image)

        if not dpkg_success:
            current_versions_in_images = {}
            for image in images_src:
                _, versions_image = c2cciutils.lib.docker.get_dpkg_packages_versions(image)
                current_versions_in_images[image] = {k: str(v) for k, v in versions_image.items()}
            if dpkg_config_found:
                print(
                    "::error::Some packages are have a greater version in the config raster then in the image."
                )
            print("Current versions of the Debian packages in Docker images:")
            print(yaml.dump(current_versions_in_images, Dumper=yaml.SafeDumper, default_flow_style=False))

            if dpkg_config_found:
                success = False

    helm_config = cast(
        c2cciutils.configuration.PublishHelmConfig,
        config.get("publish", {}).get("helm", {}) if config.get("publish", {}).get("helm", False) else {},
    )
    if helm_config and helm_config["folders"] and version_type in helm_config.get("versions", []):
        url = "https://github.com/helm/chart-releaser/releases/download/v1.2.1/chart-releaser_1.2.1_linux_amd64.tar.gz"
        response = requests.get(url, stream=True, timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")))
        with tarfile.open(fileobj=response.raw, mode="r:gz") as file:
            file.extractall(path=os.path.expanduser("~/.local/bin"))

        owner, repo = full_repo_split
        commit_sha = (
            subprocess.run(["git", "rev-parse", "HEAD"], check=True, stdout=subprocess.PIPE)
            .stdout.strip()
            .decode()
        )
        token = (
            os.environ["GITHUB_TOKEN"].strip()
            if "GITHUB_TOKEN" in os.environ
            else c2cciutils.gopass("gs/ci/github/token/gopass")
        )
        assert token is not None
        if version_type == "version_branch":
            last_tag = (
                subprocess.run(
                    ["git", "describe", "--abbrev=0", "--tags"], check=True, stdout=subprocess.PIPE
                )
                .stdout.strip()
                .decode()
            )
            expression = re.compile(r"^[0-9]+\.[0-9]+\.[0-9]+$")
            while expression.match(last_tag) is None:
                last_tag = (
                    subprocess.run(
                        ["git", "describe", "--abbrev=0", "--tags", f"{last_tag}^"],
                        check=True,
                        stdout=subprocess.PIPE,
                    )
                    .stdout.strip()
                    .decode()
                )

            versions = last_tag.split(".")
            versions[-1] = str(int(versions[-1]) + 1)
            version = ".".join(versions)

        for folder in helm_config["folders"]:
            success &= c2cciutils.publish.helm(folder, version, owner, repo, commit_sha, token)

    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3


import argparse
import configparser
import json


def main() -> None:
    """Run the command."""
    parser = argparse.ArgumentParser(
        description="""Output packages with versions from Pipfile.lock in Pipfile format (similar to pip freeze).
Useful to pin all the dependency in the Pipfile, on stabilization branch to be able to upgrade one package that have a security issue."""
    )
    parser.add_argument("--packages", action="store_true", help="Output only the packages section")
    parser.add_argument("--dev-packages", action="store_true", help="Output only the dev-packages section")
    parser.add_argument("--pipfile", default="Pipfile", help="The base Pipfile filename")
    args = parser.parse_args()

    packages = {}
    dev_packages = {}

    with open(args.pipfile + ".lock", encoding="utf-8") as pipfilelock_file:
        pipfilelock = json.loads(pipfilelock_file.read())
        for pkg, pkg_config in pipfilelock["default"].items():
            packages[pkg] = pkg_config["version"]
        for pkg, pkg_config in pipfilelock["develop"].items():
            dev_packages[pkg] = pkg_config["version"]

    config = configparser.ConfigParser()
    config.read(args.pipfile)

    if args.packages or not args.packages and not args.dev_packages:
        print("[packages]")
        print("# Lock dependencies")
        for pkg, version in packages.items():
            if pkg not in config["packages"] and f'"{pkg}"' not in config["packages"]:
                quote = '"' if "." in pkg else ""
                print(f'{quote}{pkg}{quote} = "{version}"')

    if args.packages and args.dev_packages or not args.packages and not args.dev_packages:
        print()

    if args.dev_packages or not args.packages and not args.dev_packages:
        print("[dev-packages]")
        print("# Lock dependencies")
        for pkg, version in dev_packages.items():
            if pkg not in config["dev-packages"] and f'"{pkg}"' not in config["dev-packages"]:
                quote = '"' if "." in pkg else ""
                print(f'{quote}{pkg}{quote} = "{version}"')


if __name__ == "__main__":
    main()



"""
Automatically generated file from a JSON schema.
"""

from typing import Dict, List, Literal, TypedDict, Union

# Application configuration.
#
# An application configuration
ApplicationConfiguration = TypedDict(
    "ApplicationConfiguration",
    {
        # URL pattern.
        #
        # URL pattern, to be used for files that didn't come from GitHub release, available arguments: {version}
        "url-pattern": str,
        "type": "TheTypeOfFile",
        # The filename to get.
        #
        # The filename to get in the GitHub release
        "get-file-name": str,
        # The created tile name.
        #
        # The name of the final tile name we will create
        "to-file-name": str,
        # The tile name to get in the tar file.
        "tar-file-name": str,
        # The commands to run after the tile creation.
        "finish-commands": List[List[str]],
    },
    total=False,
)


ApplicationsConfiguration = Dict[str, "ApplicationConfiguration"]
"""
Applications configuration.

All the applications configuration
"""


TheTypeOfFile = Union[Literal["tar"]]
"""
The type of file.

The type of file
"""
THETYPEOFFILE_TAR: Literal["tar"] = "tar"
"""The values for the 'The type of file' enum"""



from __future__ import unicode_literals
import re
from pkg_resources import Requirement as Req

from .fragment import get_hash_info, parse_fragment, parse_extras_require
from .vcs import VCS, VCS_SCHEMES


URI_REGEX = re.compile(
    r'^(?P<scheme>https?|file|ftps?)://(?P<path>[^#]+)'
    r'(#(?P<fragment>\S+))?'
)

VCS_REGEX = re.compile(
    r'^(?P<scheme>{0})://'.format(r'|'.join(
        [scheme.replace('+', r'\+') for scheme in VCS_SCHEMES])) +
    r'((?P<login>[^/@]+)@)?'
    r'(?P<path>[^#@]+)'
    r'(@(?P<revision>[^#]+))?'
    r'(#(?P<fragment>\S+))?'
)

# This matches just about everyting
LOCAL_REGEX = re.compile(
    r'^((?P<scheme>file)://)?'
    r'(?P<path>[^#]+)' +
    r'(#(?P<fragment>\S+))?'
)

NAME_EQ_REGEX = re.compile(r'name="(\S+)"')

PER_REQ_PARAM_REGEX = re.compile(r'\s*--[a-z-]+=\S+')

EDITABLE_REGEX = re.compile(r'^(-e|--editable=?)\s*')

class Requirement(object):
    """
    Represents a single requirement

    Typically instances of this class are created with ``Requirement.parse``.
    For local file requirements, there's no verification that the file
    exists. This class attempts to be *dict-like*.

    See: http://www.pip-installer.org/en/latest/logic.html

    **Members**:

    * ``line`` - the actual requirement line being parsed
    * ``editable`` - a boolean whether this requirement is "editable"
    * ``local_file`` - a boolean whether this requirement is a local file/path
    * ``specifier`` - a boolean whether this requirement used a requirement
      specifier (eg. "django>=1.5" or "requirements")
    * ``vcs`` - a string specifying the version control system
    * ``revision`` - a version control system specifier
    * ``name`` - the name of the requirement
    * ``uri`` - the URI if this requirement was specified by URI
    * ``subdirectory`` - the subdirectory fragment of the URI
    * ``path`` - the local path to the requirement
    * ``hash_name`` - the type of hashing algorithm indicated in the line
    * ``hash`` - the hash value indicated by the requirement line
    * ``extras`` - a list of extras for this requirement
      (eg. "mymodule[extra1, extra2]")
    * ``specs`` - a list of specs for this requirement
      (eg. "mymodule>1.5,<1.6" => [('>', '1.5'), ('<', '1.6')])
    * ``provenance`` - a tuple of (file_name, start_line, end_line), line numbers are 1-based
    """

    def __init__(self, line):
        # Do not call this private method
        self.line = line
        self.editable = False
        self.local_file = False
        self.specifier = False
        self.vcs = None
        self.name = None
        self.subdirectory = None
        self.uri = None
        self.path = None
        self.revision = None
        self.hash_name = None
        self.hash = None
        self.extras = []
        self.specs = []
        self.provenance = None

    def __repr__(self):
        return '<Requirement: "{0}">'.format(self.line)

    def __getitem__(self, key):
        return getattr(self, key)

    def keys(self):
        return self.__dict__.keys()

    @classmethod
    def parse_editable(cls, line):
        """
        Parses a Requirement from an "editable" requirement which is either
        a local project path or a VCS project URI.

        See: pip/req.py:from_editable()

        :param line: an "editable" requirement
        :returns: a Requirement instance for the given line
        :raises: ValueError on an invalid requirement
        """

        req = cls('-e {0}'.format(line))
        req.editable = True
        vcs_match = VCS_REGEX.match(line)
        local_match = LOCAL_REGEX.match(line)

        if vcs_match is not None:
            groups = vcs_match.groupdict()
            if groups.get('login'):
                req.uri = '{scheme}://{login}@{path}'.format(**groups)
            else:
                req.uri = '{scheme}://{path}'.format(**groups)
            req.revision = groups['revision']
            if groups['fragment']:
                fragment = parse_fragment(groups['fragment'])
                egg = fragment.get('egg')
                req.name, req.extras = parse_extras_require(egg)
                req.hash_name, req.hash = get_hash_info(fragment)
                req.subdirectory = fragment.get('subdirectory')
            for vcs in VCS:
                if req.uri.startswith(vcs):
                    req.vcs = vcs
        else:
            assert local_match is not None, 'This should match everything'
            groups = local_match.groupdict()
            req.local_file = True
            if groups['fragment']:
                fragment = parse_fragment(groups['fragment'])
                egg = fragment.get('egg')
                req.name, req.extras = parse_extras_require(egg)
                req.hash_name, req.hash = get_hash_info(fragment)
                req.subdirectory = fragment.get('subdirectory')
            req.path = groups['path']

        return req

    @classmethod
    def parse_line(cls, line):
        """
        Parses a Requirement from a non-editable requirement.

        See: pip/req.py:from_line()

        :param line: a "non-editable" requirement
        :returns: a Requirement instance for the given line
        :raises: ValueError on an invalid requirement
        """

        req = cls(line)

        vcs_match = VCS_REGEX.match(line)
        uri_match = URI_REGEX.match(line)
        local_match = LOCAL_REGEX.match(line)

        if vcs_match is not None:
            groups = vcs_match.groupdict()
            if groups.get('login'):
                req.uri = '{scheme}://{login}@{path}'.format(**groups)
            else:
                req.uri = '{scheme}://{path}'.format(**groups)
            req.revision = groups['revision']
            if groups['fragment']:
                fragment = parse_fragment(groups['fragment'])
                egg = fragment.get('egg')
                req.name, req.extras = parse_extras_require(egg)
                req.hash_name, req.hash = get_hash_info(fragment)
                req.subdirectory = fragment.get('subdirectory')
            for vcs in VCS:
                if req.uri.startswith(vcs):
                    req.vcs = vcs
        elif uri_match is not None:
            groups = uri_match.groupdict()
            req.uri = '{scheme}://{path}'.format(**groups)
            if groups['fragment']:
                fragment = parse_fragment(groups['fragment'])
                egg = fragment.get('egg')
                req.name, req.extras = parse_extras_require(egg)
                req.hash_name, req.hash = get_hash_info(fragment)
                req.subdirectory = fragment.get('subdirectory')
            if groups['scheme'] == 'file':
                req.local_file = True
        elif '#egg=' in line:
            # Assume a local file match
            assert local_match is not None, 'This should match everything'
            groups = local_match.groupdict()
            req.local_file = True
            if groups['fragment']:
                fragment = parse_fragment(groups['fragment'])
                egg = fragment.get('egg')
                name, extras = parse_extras_require(egg)
                req.name = fragment.get('egg')
                req.hash_name, req.hash = get_hash_info(fragment)
                req.subdirectory = fragment.get('subdirectory')
            req.path = groups['path']
        elif line.startswith('./'):
            setup_file = open(line + "/setup.py", "r")
            setup_content = setup_file.read()
            name_search = NAME_EQ_REGEX.search(setup_content)
            if name_search:
                req.name = name_search.group(1)
            req.local_file = True
        else:
            # This is a requirement specifier.
            # Delegate to pkg_resources and hope for the best
            req.specifier = True

            # an optional per-requirement param is not part of the req specifier
            #   see https://pip.pypa.io/en/stable/reference/pip_install/#per-requirement-overrides
            #   and https://pip.pypa.io/en/stable/reference/pip_install/#hash-checking-mode
            line = PER_REQ_PARAM_REGEX.sub('', line)

            pkg_req = Req.parse(line)
            req.name = pkg_req.unsafe_name
            req.extras = list(pkg_req.extras)
            req.specs = pkg_req.specs
        return req

    @classmethod
    def parse(cls, line):
        """
        Parses a Requirement from a line of a requirement file.

        :param line: a line of a requirement file
        :returns: a Requirement instance for the given line
        :raises: ValueError on an invalid requirement
        """

        if line.startswith('-e') or line.startswith('--editable'):
            # Editable installs are either a local project path
            # or a VCS project URI
            return cls.parse_editable(
                EDITABLE_REGEX.sub('', line))

        return cls.parse_line(line)



#!/usr/bin/env python3

"""
Trigger an image update on the argocd repository.
"""

import argparse
import os.path
import random
import subprocess  # nosec
import sys
from typing import List

import requests
import yaml


def main() -> None:
    """
    Trigger an image update on the argocd repository.

    Only the branch present in the HELM_RELEASE_NAMES environment variable will be considered.
    """
    parser = argparse.ArgumentParser(
        description="""Trigger an image update on the argocd repository.

    Only the branch present in the HELM_RELEASE_NAMES environment variable will be considered."""
    )
    parser.add_argument("--version", help="The version to be exported")
    parser.add_argument("--event-type", default="image-update", help="The event name to be triggered")
    parser.add_argument(
        "--repository",
        default="camptocamp/argocd-gs-platform-ch-development-apps",
        help="The repository name to be triggered",
    )

    args = parser.parse_args()

    if args.version:
        version = args.version
    else:
        ref = os.environ["GITHUB_REF"].split("/")

        if ref[1] != "heads":
            print("::error::Not a branch")
            sys.exit(0)

        version = "/".join(ref[2:])

        if version not in os.environ.get("HELM_RELEASE_NAMES", "").split(","):
            print("::error::Not a release branch")
            sys.exit(0)

    images_full = []
    with open("ci/config.yaml", encoding="utf-8") as config_file:
        ci_config = yaml.load(config_file, Loader=yaml.SafeLoader)
        for image_config in ci_config.get("publish", {}).get("docker", {}).get("images", []):
            images_full.append(image_config["name"])

    dispatch(args.repository, args.event_type, [f"{image}:{version}" for image in images_full])


def dispatch(repository: str, event_type: str, images_full: List[str]) -> None:
    """
    Trigger an image update on the argocd repository.
    """

    id_ = random.randint(1, 100000)  # nosec
    print(f"Triggering {event_type}:{id_} on {repository} with {','.join(images_full)}")

    response = requests.post(
        f"https://api.github.com/repos/{repository}/dispatches",
        headers={
            "Content-Type": "application/json2",
            "Accept": "application/vnd.github.v3+json",
            "Authorization": "token "
            + subprocess.run(
                ["gopass", "show", "gs/ci/github/token/gopass"], check=True, stdout=subprocess.PIPE
            )
            .stdout.decode()
            .strip(),
        },
        json={"event_type": event_type, "client_payload": {"name": " ".join(images_full), "id": id_}},
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    )
    response.raise_for_status()


if __name__ == "__main__":
    main()



"""
The pull request checking functions.

Commits, messages and labels.
"""

import os
import re
import subprocess  # nosec
from tempfile import NamedTemporaryFile
from typing import Any, Dict, List, Optional

import requests
import yaml

import c2cciutils.configuration


def _commit_intro(need_separator: bool, commit: Dict[str, Any]) -> bool:
    head = commit["commit"]["message"].split("\n")[0]
    if need_separator:
        print("-" * 30)
    print(f'{commit["commit"]["tree"]["sha"]}: {head}')
    return True


def print_event(github_event: Dict[str, Any], **kwargs: Any) -> bool:
    """
    Print the github object.
    """
    del kwargs
    print(yaml.dump(github_event, default_flow_style=False, Dumper=yaml.SafeDumper))
    return True


def commits_messages(
    config: c2cciutils.configuration.PullRequestChecksCommitsMessagesConfiguration,
    commits: List[Dict[str, Any]],
    **kwargs: Any,
) -> bool:
    """
    Check the commits messages.

    - They should start with a capital letter.
    - They should not be too short.
    - They should not be a squash or fixup commit.
    - They should not be a merge commit.
    - They should not be a revert commit.
    """
    del kwargs

    need_separator = False
    success = True
    first_capital = re.compile(r"^[^a-z]")
    commit_hash = set()
    for commit in commits:
        need_head = True
        commit_hash.add(commit["sha"])
        message_lines = commit["commit"]["message"].split("\n")
        head = message_lines[0]
        if config.get(
            "check_fixup", c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_FIXUP_DEFAULT
        ) and head.startswith("fixup! "):
            if need_head:
                need_separator = _commit_intro(need_separator, commit)
            need_head = False
            print("::error::Fixup message not allowed")
            success = False
        if config.get(
            "check_squash", c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_SQUASH_DEFAULT
        ) and head.startswith("squash! "):
            if need_head:
                need_separator = _commit_intro(need_separator, commit)
            need_head = False
            print("::error::Squash message not allowed")
            success = False
        if (
            config.get(
                "check_first_capital",
                c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_FIRST_CAPITAL_DEFAULT,
            )
            and first_capital.match(head) is None
        ):
            if need_head:
                need_separator = _commit_intro(need_separator, commit)
            need_head = False
            print("::error::The first letter of message head should be a capital")
            success = False
        min_length = config.get(
            "min_head_length",
            c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_MIN_HEAD_LENGTH_DEFAULT,
        )
        if min_length > 0 and len(head) < min_length:
            if need_head:
                need_separator = _commit_intro(need_separator, commit)
            need_head = False
            print(f"The message head should be at least {min_length} characters long")
            success = False
        if (
            config.get(
                "check_no_merge_commits",
                c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_NO_MERGE_COMMITS_DEFAULT,
            )
            and len(commit["parents"]) != 1
        ):
            if need_head:
                need_separator = _commit_intro(need_separator, commit)
            need_head = False
            print("::error::The merge commit are not allowed")
            success = False
        if config.get(
            "check_no_own_revert",
            c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_NO_OWN_REVERT_DEFAULT,
        ) and (
            head.startswith("Revert ")
            and len(message_lines) == 3
            and message_lines[2].startswith("This reverts commit ")
        ):
            revert_commit_hash = message_lines[2][len("This reverts commit ") : -1]
            if revert_commit_hash in commit_hash:
                if need_head:
                    need_separator = _commit_intro(need_separator, commit)
                need_head = False
                print(f"Revert own commits is not allowed ({revert_commit_hash})")
                success = False
                continue
    return success


def commits_spell(
    config: c2cciutils.configuration.PullRequestChecksCommitsSpellingConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    commits: List[Dict[str, Any]],
    **kwargs: Any,
) -> bool:
    """Check the spelling of the commits body."""
    del kwargs

    spellcheck_cmd = c2cciutils.get_codespell_command(full_config)

    success = True
    need_separator = False
    for commit in commits:
        with NamedTemporaryFile("w+t", encoding="utf-8", suffix=".yaml") as temp_file:
            if config.get(
                "only_head", c2cciutils.configuration.PULL_REQUEST_CHECKS_COMMITS_MESSAGES_ONLY_HEAD_DEFAULT
            ):
                head = commit["commit"]["message"].split("\n")[0]
                temp_file.write(head)
            else:
                temp_file.write(commit["commit"]["message"])
            temp_file.flush()
            spell = subprocess.run(  # nosec # pylint: disable=subprocess-run-check
                spellcheck_cmd + [temp_file.name], stdout=subprocess.PIPE, stderr=subprocess.PIPE
            )
            if spell.returncode != 0:
                need_separator = _commit_intro(need_separator, commit)
                print("::error::Code spell error")
                print(spell.stderr)
                print(spell.stdout)
                success = False
    return success


def pull_request_spell(
    config: c2cciutils.configuration.PullRequestChecksPullRequestSpellingConfiguration,
    full_config: c2cciutils.configuration.Configuration,
    github_event: Dict[str, Any],
    **kwargs: Any,
) -> bool:
    """Check the spelling of the pull request title and message."""
    del kwargs

    spellcheck_cmd = c2cciutils.get_codespell_command(full_config)

    with NamedTemporaryFile("w+t") as temp_file:
        temp_file.write(github_event["event"]["pull_request"]["title"])
        temp_file.write("\n")
        if not config.get(
            "only_head", c2cciutils.configuration.PULL_REQUEST_CHECKS_ONLY_HEAD_DEFAULT
        ) and github_event["event"]["pull_request"].get("body"):
            temp_file.write("\n")
            temp_file.write(github_event["event"]["pull_request"]["body"])
            temp_file.write("\n")
        temp_file.flush()
        spell = subprocess.run(  # nosec # pylint: disable=subprocess-run-check
            spellcheck_cmd + [temp_file.name], stdout=subprocess.PIPE, stderr=subprocess.PIPE
        )
        if spell.returncode != 0:
            print("::error::Code spell error in pull request")
            print(spell.stderr)
            print(spell.stdout)
            return False
    return True


def pull_request_labels(github_event: Dict[str, Any], **kwargs: Any) -> bool:
    """Check it the label are set correctly for the changelog generation."""
    del kwargs

    if github_event["actor"] == "renovate[bot]":
        return True

    if not os.path.exists(".github/changelog-config.yaml"):
        return True

    required_labels = []
    with open(".github/changelog-config.yaml", encoding="utf-8") as changelog_config_file:
        changelog_config = yaml.load(changelog_config_file, Loader=yaml.SafeLoader)
        for section in changelog_config.values():
            if "labels" in section:
                required_labels.extend(section["labels"])

    print(f"Required one onf the following labels: {', '.join(required_labels)}")

    if required_labels:
        labels = [
            label["name"]
            for label in github_event["event"]["pull_request"]["labels"]
            if label["name"] in required_labels
        ]
        if len(labels) == 0:
            print(f"::error::No required label found: {', '.join(required_labels)}")
            return False
        if len(labels) > 1:
            print(f"::error::Too many required labels found: {', '.join(labels)}")
            return False
    return True


GET_ISSUE_RE = [
    re.compile(r"^([A-Z]{3,6}-[0-9]+)-.*$"),
    re.compile(r"^([a-z]{3,6}-[0-9]+)-.*$"),
    re.compile(r"^.*-([A-Z]{3,6}-[0-9]+)$"),
    re.compile(r"^.*-([a-z]{3,6}-[0-9]+)$"),
]


def _get_issue_number(name: str) -> Optional[str]:
    for re_ in GET_ISSUE_RE:
        match = re_.match(name)
        if match is not None:
            return match.group(1)
    return None


def add_issue_link(github_event: Dict[str, Any], **kwargs: Any) -> bool:
    """Add a comment with the link to Jira if needed."""
    del kwargs

    issue_number = _get_issue_number(github_event["event"]["pull_request"]["head"]["ref"])

    if issue_number is None:
        return True

    issue_number = issue_number.upper()

    body = github_event["event"]["pull_request"].get("body") or ""
    if issue_number in body.upper():
        return True

    comments_response = requests.get(
        github_event["event"]["pull_request"]["_links"]["comments"]["href"],
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
        headers=c2cciutils.add_authorization_header({}),
    )
    c2cciutils.check_response(comments_response)
    comments = comments_response.json()

    for comment in comments:
        if issue_number in comment.get("body", "").upper():
            return True

    response = requests.post(
        github_event["event"]["pull_request"]["_links"]["comments"]["href"],
        headers={
            "Accept": "application/vnd.github+json",
            "Authorization": f"Bearer {github_event['token']}",
        },
        json={"body": f"See also: [{issue_number}](https://jira.camptocamp.com/browse/{issue_number})"},
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    )

    if not response.ok:
        print(f"Unable to add the comment: {response.text}")
    return response.ok



#!/usr/bin/env python3

import argparse
import os
import subprocess  # nosec
import sys
from datetime import datetime


def _print(message: str) -> None:
    print(message)
    sys.stdout.flush()


def main() -> None:
    """Print the list of running docker containers and their logs formatted for GitHub CI."""
    parser = argparse.ArgumentParser(
        description=("Print the list of running docker containers and their logs formatted for GitHub CI.")
    )
    parser.parse_args()

    if os.path.exists("docker-compose.yaml"):
        _print("::group::Docker Compose ps")
        subprocess.run(["docker", "compose", "ps"], check=False)
        _print("::endgroup::")

    _print("::group::Docker ps")
    subprocess.run(["docker", "ps"], check=False)
    _print("::endgroup::")

    # Store in /tmp/docker-logs-timestamp the current timestamp to avoid printing same logs multiple times.
    timestamp_args = []
    if os.path.exists("/tmp/docker-logs-timestamp"):
        with open("/tmp/docker-logs-timestamp", encoding="utf-8") as timestamp_file:
            timestamp_args = [f"--since={timestamp_file.read().strip()}Z"]

    with open("/tmp/docker-logs-timestamp", "w", encoding="utf-8") as timestamp_file:
        timestamp_file.write(datetime.utcnow().isoformat())

    for name in (
        subprocess.run(
            ["docker", "ps", "--all", "--format", "{{ .Names }}"], check=True, stdout=subprocess.PIPE
        )
        .stdout.decode()
        .split("\n")
    ):
        if name:
            _print(f"::group::{name}: New logs")
            subprocess.run(["docker", "logs"] + timestamp_args + [name], check=False)
            _print("::endgroup::")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
The pull request checker main function.
"""

import argparse
import json
import os
import sys
import traceback

import requests

import c2cciutils
import c2cciutils.pr_checks


def main() -> None:
    """
    Run the checks.
    """
    parser = argparse.ArgumentParser(description="Run the pull request checks of c2cciutils.")
    parser.add_argument("--stop", action="store_true", help="stop on first error")
    parser.add_argument("--check", help="runs only the specified check")

    args = parser.parse_args()

    github_event = json.loads(os.environ["GITHUB_EVENT"])

    commits_response = requests.get(
        github_event["event"]["pull_request"]["_links"]["commits"]["href"],
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
        headers=c2cciutils.add_authorization_header({}),
    )
    c2cciutils.check_response(commits_response)
    commits = commits_response.json()

    full_config = c2cciutils.get_config()
    config = full_config["pr-checks"]

    check_args = {
        "args": args,
        "full_config": full_config,
        "commits": commits,
        "github_event": github_event,
    }

    success = True
    for key, conf in config.items():
        if conf is not False and (args.check is None or args.check == key):
            check = getattr(c2cciutils.pr_checks, key)
            print(f"::group::Run check {key}")
            try:
                if not check(config={} if conf is True else conf, **check_args):
                    success = False
                    print("::endgroup::")
                    if args.stop:
                        sys.exit(1)
                    print("::error::With error")
                else:
                    print("::endgroup::")
            except Exception:  # pylint: disable=broad-except
                traceback.print_exc()
                success = False
                print("::endgroup::")
                if args.stop:
                    sys.exit(1)
                print("::error::With error")
    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



"""
c2cciutils shared utils function.
"""

import glob
import json
import os.path
import pkgutil
import re
import subprocess  # nosec
import sys
from typing import Any, Dict, List, Match, Optional, Pattern, Tuple, TypedDict, cast

import jsonschema_gentypes.validate
import magic
import requests
import ruamel.yaml

import c2cciutils.configuration


def get_repository() -> str:
    """
    Get the current GitHub repository like `organization/project`.
    """

    if "GITHUB_REPOSITORY" in os.environ:
        return os.environ["GITHUB_REPOSITORY"]

    remote_lines = subprocess.check_output(["git", "remote", "--verbose"]).decode().split("\n")
    remote_match = (
        re.match(r".*git@github.com:(.*).git .*", remote_lines[0]) if len(remote_lines) >= 1 else None
    )

    if remote_match:
        return remote_match.group(1)

    print("::warning::The GitHub repository isn't found, using 'camptocamp/project'")

    return "camptocamp/project"


def merge(default_config: Any, config: Any) -> Any:
    """
    Deep merge the dictionaries (on dictionaries only, not on arrays).

    Arguments:
        default_config: The default config that will be applied
        config: The base config, will be modified
    """

    if not isinstance(default_config, dict) or not isinstance(config, dict):
        return config

    for key in default_config.keys():
        if key not in config:
            config[key] = default_config[key]
        else:
            merge(default_config[key], config[key])
    return config


def get_master_branch(repo: List[str]) -> Tuple[str, bool]:
    """Get the name of the master branch."""
    master_branch = "master"
    success = False
    try:
        default_branch_json = graphql(
            "default_branch.graphql", {"name": repo[1], "owner": repo[0]}, default=False
        )
        success = default_branch_json is not False
        master_branch = default_branch_json["repository"]["defaultBranchRef"]["name"] if success else "master"
    except RuntimeError as runtime_error:
        print(runtime_error)
        print("::warning::Fallback to master")
    return master_branch, success


def get_config() -> c2cciutils.configuration.Configuration:
    """
    Get the configuration, with project and auto detections.
    """

    config: c2cciutils.configuration.Configuration = {}
    if os.path.exists("ci/config.yaml"):
        with open("ci/config.yaml", encoding="utf-8") as open_file:
            yaml_ = ruamel.yaml.YAML()
            config = yaml_.load(open_file)

    repository = get_repository()
    repo = repository.split("/")
    master_branch, _ = get_master_branch(repo)

    merge(
        {
            "version": {
                "tag_to_version_re": [
                    {"from": r"([0-9]+.[0-9]+.[0-9]+)", "to": r"\1"},
                ],
                "branch_to_version_re": [
                    {"from": r"([0-9]+.[0-9]+)", "to": r"\1"},
                    {"from": master_branch, "to": master_branch},
                ],
            }
        },
        config,
    )

    has_docker_files = bool(
        subprocess.run(
            ["git", "ls-files", "*/Dockerfile*", "Dockerfile*"], stdout=subprocess.PIPE, check=True
        ).stdout
    )
    has_python_package = bool(
        subprocess.run(
            ["git", "ls-files", "setup.py", "*/setup.py"], stdout=subprocess.PIPE, check=True
        ).stdout
    ) or bool(
        subprocess.run(
            ["git", "ls-files", "pyproject.toml", "*/pyproject.toml"], stdout=subprocess.PIPE, check=True
        ).stdout
    )

    publish_config = merge(c2cciutils.configuration.PUBLISH_DEFAULT, {})
    publish_config["pypi"]["packages"] = [{"path": "."}] if has_python_package else []
    publish_config["docker"]["images"] = [{"name": get_repository()}] if has_docker_files else []
    publish_config["helm"]["folders"] = [
        os.path.dirname(f) for f in glob.glob("./**/Chart.yaml", recursive=True)
    ]

    default_config = {
        "publish": publish_config,
        "checks": c2cciutils.configuration.CHECKS_DEFAULT,
        "pr-checks": c2cciutils.configuration.PULL_REQUEST_CHECKS_DEFAULT,
        "audit": c2cciutils.configuration.AUDIT_DEFAULT,
    }
    merge(default_config, config)

    return validate_config(config, "ci/config.yaml")


def validate_config(
    config: c2cciutils.configuration.Configuration, config_file: str
) -> c2cciutils.configuration.Configuration:
    """
    Validate the configuration.

    Arguments:
        config: The configuration to be validated
        config_file: The configuration file name, used to build the error messages

    Return the configuration (used to be chained)
    Print an message and eventually exit on validation error.
    """
    schema_data = pkgutil.get_data("c2cciutils", "schema.json")
    assert schema_data is not None

    errors, data = jsonschema_gentypes.validate.validate(
        config_file, cast(Dict[str, Any], config), json.loads(schema_data)
    )

    if errors:
        formated_errors = "\n".join(errors)
        print(f"The config file is invalid:\n{formated_errors}")
        if os.environ.get("IGNORE_CONFIG_ERROR", "FALSE").lower() != "true":
            sys.exit(1)

    return cast(c2cciutils.configuration.Configuration, data)


def error(
    checker: str,
    message: str,
    file: Optional[str] = None,
    line: Optional[int] = None,
    col: Optional[int] = None,
    error_type: str = "error",
) -> None:
    """
    Write an error or warn message formatted for GitHub if the CI environment variable is true else for IDE.

    GitHub: ::(error|warning) file=<file>,line=<line>,col=<col>:: <checker>: <message>
    IDE: [(error|warning)] <file>:<line>:<col>: <checker>: <message>

    See: https://docs.github.com/en/free-pro-team@latest/actions/reference/ \
        workflow-commands-for-github-actions#setting-an-error-message

    Arguments:
        checker: The check name, used to prefix the message
        message: The message
        file: The file where the error happens
        line: The line number of the error
        col: The column number of the error
        error_type: The kind of error (error or warning)
    """
    result = ""
    on_ci = os.environ.get("CI", "false").lower() == "true"
    if file is not None:
        result += ("file={}" if on_ci else "{}").format(file)
        if line is not None:
            result += (",line={}" if on_ci else ":{}").format(line)
            if col is not None:
                result += (",col={}" if on_ci else ":{}").format(col)
    result += (":: {}: {}" if on_ci else ": {}: {}").format(checker, message)
    if on_ci:
        # Make the error visible on GitHub workflow logs
        print(result)
        # Make the error visible as annotation
        print(f"::{error_type} {result}")
    else:
        print(f"[{error_type}] {result}")


VersionTransform = TypedDict(
    "VersionTransform",
    {
        # The from regular expression
        "from": Pattern[str],
        # The expand regular expression: https://docs.python.org/3/library/re.html#re.Match.expand
        "to": str,
    },
    total=False,
)


def compile_re(config: c2cciutils.configuration.VersionTransform, prefix: str = "") -> List[VersionTransform]:
    """
    Compile the from as a regular expression of a dictionary of the config list.

    to be used with convert and match

    Arguments:
        config: The transform config
        prefix: The version prefix

    Return the compiled transform config.
    """
    result = []
    for conf in config:
        new_conf = cast(VersionTransform, dict(conf))

        from_re = conf.get("from", r"(.*)")
        if from_re[0] == "^":
            from_re = from_re[1:]
        if from_re[-1] != "$":
            from_re += "$"
        from_re = f"^{re.escape(prefix)}{from_re}"

        new_conf["from"] = re.compile(from_re)
        result.append(new_conf)
    return result


def match(
    value: str, config: List[VersionTransform]
) -> Tuple[Optional[Match[str]], Optional[VersionTransform], str]:
    """
    Get the matched version.

    Arguments:
        value: That we want to match with
        config: The result of `compile`

    Returns the re match object, the matched config and the value as a tuple
    On no match it returns None, value
    """
    for conf in config:
        matched = conf["from"].match(value)
        if matched is not None:
            return matched, conf, value
    return None, None, value


def does_match(value: str, config: List[VersionTransform]) -> bool:
    """
    Check if the version match with the config patterns.

    Arguments:
        value: That we want to match with
        config: The result of `compile`

    Returns True it it does match else False
    """
    matched, _, _ = match(value, config)
    return matched is not None


def get_value(matched: Optional[Match[str]], config: Optional[VersionTransform], value: str) -> str:
    """
    Get the final value.

    `match`, `config` and `value` are the result of `match`.

    The `config` should have a `to` key with an expand template.

    Arguments:
        matched: The matched object to a regular expression
        config: The result of `compile`
        value: The default value on returned no match

    Return the value
    """
    return matched.expand(config.get("to", r"\1")) if matched is not None and config is not None else value


def print_versions(config: c2cciutils.configuration.PrintVersions) -> bool:
    """
    Print some tools version.

    Arguments:
        config: The print configuration
    """

    for version in config.get("versions", []):
        try:
            sys.stdout.flush()
            sys.stderr.flush()
            current_version = subprocess.check_output(version.get("cmd", [])).decode()
            print(f"{version.get('prefix', '')}{current_version}")
        except PermissionError as exception:
            error(
                "print_version",
                f"{version.get('name')}: not allowed cmd: {exception}",
                error_type="warning",
            )
        except subprocess.CalledProcessError as exception:
            error(
                "print_version",
                f"{version.get('name')}: no present: {exception}",
                error_type="warning",
            )
        except FileNotFoundError as exception:
            error(
                "print_version",
                f"{version.get('name')}: no present: {exception}",
                error_type="warning",
            )

    return True


def gopass(key: str, default: Optional[str] = None) -> Optional[str]:
    """
    Get a value from gopass.

    Arguments:
        key: The key to get
        default: the value to return if gopass is not found

    Return the value
    """
    try:
        return subprocess.check_output(["gopass", "show", key]).strip().decode()
    except FileNotFoundError:
        if default is not None:
            return default
        raise


def gopass_put(secret: str, key: str) -> None:
    """
    Put an entry in gopass.

    Arguments:
        secret: The secret value
        key: The key
    """
    subprocess.check_output(["gopass", "insert", "--force", key], input=secret.encode())


def add_authorization_header(headers: Dict[str, str]) -> Dict[str, str]:
    """
    Add the Authorization header needed to be authenticated on GitHub.

    Arguments:
        headers: The headers

    Return the headers (to be chained)
    """
    try:
        token = (
            os.environ["GITHUB_TOKEN"].strip()
            if "GITHUB_TOKEN" in os.environ
            else gopass("gs/ci/github/token/gopass")
        )
        headers["Authorization"] = f"Bearer {token}"
        return headers
    except FileNotFoundError:
        return headers


def check_response(response: requests.Response, raise_for_status: bool = True) -> Any:
    """
    Check the response and raise an exception if it's not ok.

    Also print the X-Ratelimit- headers to get information about the rate limiting.
    """
    for header in response.headers:
        if header.lower().startswith("x-ratelimit-"):
            print(f"{header}: {response.headers[header]}")
    if raise_for_status:
        response.raise_for_status()


def graphql(query_file: str, variables: Dict[str, Any], default: Any = None) -> Any:
    """
    Get a graphql result from GitHub.

    Arguments:
        query_file: Relative path from this file to the GraphQL query file.
        variables: The query variables
        default:  The return result if we are not authorized to get the resource

    Return the data result
    In case of error it throw an exception
    """

    with open(os.path.join(os.path.dirname(__file__), query_file), encoding="utf-8") as query_open:
        query = query_open.read()

    http_response = requests.post(
        os.environ.get("GITHUB_GRAPHQL_URL", "https://api.github.com/graphql"),
        data=json.dumps(
            {
                "query": query,
                "variables": variables,
            }
        ),
        headers=add_authorization_header(
            {
                "Content-Type": "application/json",
            }
        ),
        timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
    )
    if http_response.status_code in (401, 403) and default is not None:
        print(f"::warning::GraphQL error: {http_response.status_code}, use default value")
        check_response(http_response, False)
        return default
    check_response(http_response)
    json_response = http_response.json()

    if "errors" in json_response:
        raise RuntimeError(f"GraphQL error: {json.dumps(json_response['errors'], indent=2)}")
    if "data" not in json_response:
        raise RuntimeError(f"GraphQL no data: {json.dumps(json_response, indent=2)}")
    return cast(Dict[str, Any], json_response["data"])


def get_git_files_mime(
    mime_type: Optional[List[str]] = None,
    extensions: Optional[List[str]] = None,
    ignore_patterns_re: Optional[List[str]] = None,
) -> List[str]:
    """
    Get list of paths from git with all the files that have the specified mime type.

    Arguments:
        mime_type: The considered MIME type
        extensions: The considered extensions
        ignore_patterns_re: A list of regular expressions of files that we should ignore
    """
    if mime_type is None:
        mime_type = ["text/x-python", "text/x-script.python"]
    if extensions is None:
        extensions = [".py"]
    ignore_patterns_compiled = [re.compile(p) for p in ignore_patterns_re or []]
    result = []

    for filename in subprocess.check_output(["git", "ls-files"]).decode().strip().split("\n"):
        if os.path.isfile(filename) and (
            os.path.splitext(filename)[1] in extensions or magic.from_file(filename, mime=True) in mime_type
        ):
            accept = True
            for pattern in ignore_patterns_compiled:
                if pattern.search(filename):
                    accept = False
                    break
            if accept:
                result.append(filename)
    return result


def get_branch(branch: Optional[str], master_branch: str = "master") -> str:
    """
    Get the branch name.

    Arguments:
        branch: The forced to use branch name
        master_branch: The master branch name, can be used as default value

    Return the branch name
    """

    if branch is not None:
        return branch
    try:
        branch = (
            subprocess.run(["git", "rev-parse", "--abbrev-ref", "HEAD"], check=True, stdout=subprocess.PIPE)
            .stdout.decode()
            .strip()
        )
    except subprocess.CalledProcessError as exception:
        print(f"Error getting branch: {exception}")
        branch = "HEAD"

    if branch == "HEAD":
        branch = os.environ.get("GITHUB_HEAD_REF", master_branch)
        assert branch is not None
    return branch


def get_based_on_master(
    repo: List[str],
    override_current_branch: Optional[str],
    master_branch: str,
    config: c2cciutils.configuration.Configuration,
) -> bool:
    """
    Check that we are not on a release branch (to avoid errors in versions check).

    This function will check the last 20 commits in current branch,
    and for each other branch (max 50) check if any commit in last 10 commits is the current one.

    Arguments:
        repo: The repository [<organization>, <name>]
        override_current_branch: The branch to use instead of the current one
        master_branch: The master branch name
        config: The full configuration
    """
    if os.environ.get("GITHUB_REF", "").startswith("refs/tags/"):
        # The tags are never consider as based on master
        return False
    current_branch = get_branch(override_current_branch, master_branch)
    if current_branch == master_branch:
        return True
    branches_re = compile_re(config["version"].get("branch_to_version_re", []))
    if does_match(current_branch, branches_re):
        return False
    if os.environ.get("GITHUB_BASE_REF"):
        return os.environ.get("GITHUB_BASE_REF") == master_branch
    commits_repository_json = graphql(
        "commits.graphql", {"name": repo[1], "owner": repo[0], "branch": current_branch}
    ).get("repository", {})
    commits_json = (
        commits_repository_json.get("ref", {}).get("target", {}).get("history", {}).get("nodes", [])
        if commits_repository_json.get("ref")
        else []
    )
    branches_json = [
        branch
        for branch in (
            graphql("branches.graphql", {"name": repo[1], "owner": repo[0]})["repository"]["refs"]["nodes"]
        )
        if branch["name"] != current_branch and does_match(branch["name"], branches_re)
    ]
    based_branch = master_branch
    found = False
    for commit in commits_json:
        for branch in branches_json:
            commits = [
                branch_commit
                for branch_commit in branch["target"]["history"]["nodes"]
                if commit["oid"] == branch_commit["oid"]
            ]
            if commits:
                based_branch = branch["name"]
                found = True
                break
        if found:
            break
    return based_branch == master_branch


def get_codespell_command(config: c2cciutils.configuration.Configuration, fix: bool = False) -> List[str]:
    """
    Get the codespell command.

    Arguments:
        config: The full configuration
        fix: If we should fix the errors
    """
    codespell_config = config.get("checks", {}).get("codespell", {})
    codespell_config = codespell_config if isinstance(codespell_config, dict) else {}
    command = ["codespell"]
    if fix:
        command.append("--write-changes")
    if os.path.exists("spell-ignore-words.txt"):
        command.append("--ignore-words=spell-ignore-words.txt")
    dictionaries = codespell_config.get(
        "internal_dictionaries", c2cciutils.configuration.CODESPELL_DICTIONARIES_DEFAULT
    )
    if dictionaries:
        command.append("--builtin=" + ",".join(dictionaries))
    command += codespell_config.get("arguments", c2cciutils.configuration.CODESPELL_ARGUMENTS_DEFAULT)
    return command


def snyk_exec() -> Tuple[str, Dict[str, str]]:
    """Get the Snyk cli executable path."""
    env = {**os.environ}
    env["FORCE_COLOR"] = "true"
    if "SNYK_TOKEN" not in env:
        token = gopass("gs/ci/snyk/token")
        if token is not None:
            env["SNYK_TOKEN"] = token
    if "SNYK_ORG" in env:
        subprocess.run(["snyk", "config", "set", f"org={env['SNYK_ORG']}"], check=True, env=env)

    return os.path.join(os.path.dirname(os.path.abspath(__file__)), "node_modules/snyk/bin/snyk"), env



#!/usr/bin/env python3

import argparse
import os
import subprocess  # nosec
import tarfile
import urllib
from glob import glob
from io import BytesIO
from typing import Dict, cast

import requests
import yaml

from c2cciutils import applications_definition


def main() -> None:
    """Download applications from GitHub releases or any other URLs to the ~/.local/bin folder."""

    argparser = argparse.ArgumentParser(
        description="""Download applications from GitHub releases or any other URLs to the ~/.local/bin folder.
            Based on tow files, the first contains the information about from where to download the applications,
            how to extract the application from the archive, and the executable name.
            The second file contains the versions of the applications to download,
            this file is usually updated by Renovate."""
    )
    argparser.add_argument("--applications-file", required=True)
    argparser.add_argument("--versions-file", required=True)
    args = argparser.parse_args()

    with open(args.versions_file, "r", encoding="utf-8") as config_file:
        versions = cast(Dict[str, str], yaml.load(config_file, Loader=yaml.SafeLoader))
    with open(args.applications_file, "r", encoding="utf-8") as config_file:
        applications = cast(
            applications_definition.ApplicationsConfiguration, yaml.load(config_file, Loader=yaml.SafeLoader)
        )
    download_applications(applications, versions)


def download_c2cciutils_applications() -> None:
    """Download the applications defined in the c2cciutils package."""
    with open(
        os.path.join(os.path.dirname(os.path.dirname(__file__)), "applications-versions.yaml"),
        "r",
        encoding="utf-8",
    ) as config_file:
        versions = cast(Dict[str, str], yaml.load(config_file, Loader=yaml.SafeLoader))
    with open(
        os.path.join(os.path.dirname(os.path.dirname(__file__)), "applications.yaml"), "r", encoding="utf-8"
    ) as config_file:
        applications = cast(
            applications_definition.ApplicationsConfiguration, yaml.load(config_file, Loader=yaml.SafeLoader)
        )
    download_applications(applications, versions)


def download_applications(
    applications: applications_definition.ApplicationsConfiguration, versions: Dict[str, str]
) -> None:
    """Download the versions of applications specified in the configuration."""

    bin_path = os.path.join(os.environ["HOME"], ".local", "bin")
    if not os.path.exists(bin_path):
        os.makedirs(bin_path)

    for key, app in applications.items():
        # The versions file is used to don't re-download an already downloaded application
        version_file = os.path.join(bin_path, f"{app['to-file-name']}-version-{versions[key]}")
        if not os.path.exists(version_file):
            print(f"Download {app['to-file-name']} version {versions[key]}")
            version = versions[key]
            version_quote = urllib.parse.quote_plus(version)
            params = {
                "version": version,
                "version_quote": version_quote,
                "short_version": version.lstrip("v"),
            }
            response = requests.get(
                app.get(
                    "url-pattern",
                    f"https://github.com/{key}/releases/download/{version_quote}/{app.get('get-file-name', '')}",
                ).format(**params),
                timeout=int(os.environ.get("C2CCIUTILS_TIMEOUT", "30")),
            )
            response.raise_for_status()

            if app.get("type") == "tar":
                with tarfile.open(fileobj=BytesIO(response.content)) as tar:
                    extracted_file = tar.extractfile(app["tar-file-name"])
                    assert extracted_file is not None
                    content = extracted_file.read()
            else:
                content = response.content

            with open(os.path.join(bin_path, app["to-file-name"]), "wb") as destination_file:
                destination_file.write(content)

            for command in app.get("finish-commands", []):
                subprocess.run(command, check=True, cwd=bin_path)

            for file_name in glob(f"{app['to-file-name']}-version-*"):
                os.remove(file_name)
            with open(version_file, "w", encoding="utf-8") as _:
                pass


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
The audit main function.
"""

import argparse
import os
import sys

import c2cciutils.audit


def main() -> None:
    """
    Run the audit.
    """
    parser = argparse.ArgumentParser(description="Run the audit of c2cciutils.")
    parser.add_argument("--branch", help="The branch to audit, not defined means autodetect")
    parser.add_argument("--check", help="Runs only the specified check")
    parser.add_argument("--fix", action="store_true", help="Fix issues")

    args = parser.parse_args()

    full_config = c2cciutils.get_config()
    config = full_config.get("audit", {})
    success = True
    for key, conf in config.items():
        if conf is not False and (args.check is None or args.check == key):
            audit = getattr(c2cciutils.audit, key)
            print(f"Run audit {key}")
            success &= audit({} if conf is True else conf, full_config, args)
    if not success and os.environ.get("TEST") != "TRUE":
        sys.exit(1)


if __name__ == "__main__":
    main()



"""
Automatically generated file from a JSON schema.
"""

from typing import Any, Dict, List, Literal, TypedDict, Union

AUDIT_DEFAULT = {
    "print_versions": {
        "versions": [
            {"name": "c2cciutils", "cmd": ["c2cciutils", "--version"]},
            {"name": "python", "cmd": ["python3", "--version"]},
            {"name": "node", "prefix": "node ", "cmd": ["node", "--version"]},
            {"name": "npm", "prefix": "npm ", "cmd": ["npm", "--version"]},
        ]
    },
    "snyk": True,
    "outdated_versions": True,
}
"""Default value of the field path 'configuration audit'"""


AUDIT_SNYK_FILES_NO_INSTALL_DEFAULT: List[Any] = []
"""Default value of the field path 'Audit snyk config files_no_install'"""


AUDIT_SNYK_FIX_ARGUMENTS_DEFAULT = ["--all-projects"]
"""Default value of the field path 'Audit snyk config fix_arguments'"""


AUDIT_SNYK_FIX_PULL_REQUEST_ARGUMENTS_DEFAULT = ["--fill", "--label=dependencies"]
"""Default value of the field path 'Audit snyk config fix_github_create_pull_request_arguments'"""


AUDIT_SNYK_MONITOR_ARGUMENTS_DEFAULT = ["--all-projects"]
"""Default value of the field path 'Audit snyk config monitor_arguments'"""


AUDIT_SNYK_PIPENV_SYNC_ARGUMENTS_DEFAULT: List[Any] = []
"""Default value of the field path 'Audit snyk config pipenv_sync_arguments'"""


AUDIT_SNYK_PIP_INSTALL_ARGUMENTS_DEFAULT = ["--user"]
"""Default value of the field path 'Audit snyk config pip_install_arguments'"""


AUDIT_SNYK_TEST_ARGUMENTS_DEFAULT = ["--all-projects", "--fail-on=all", "--severity-threshold=medium"]
"""Default value of the field path 'Audit snyk config test_arguments'"""


class Audit(TypedDict, total=False):
    """
    Audit.

    The audit configuration

    default:
      outdated_versions: true
      print_versions:
        versions:
        - cmd:
          - c2cciutils
          - --version
          name: c2cciutils
        - cmd:
          - python3
          - --version
          name: python
        - cmd:
          - node
          - --version
          name: node
          prefix: 'node '
        - cmd:
          - npm
          - --version
          name: npm
          prefix: 'npm '
      snyk: true
    """

    outdated_versions: "AuditOutdatedVersions"
    snyk: "AuditWithSnyk"
    print_versions: "PrintVersions"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """


AuditOutdatedVersions = bool
"""
Audit outdated versions.

Audit of outdated version
"""


class AuditSnykConfig(TypedDict, total=False):
    """
    Audit snyk config.

    The audit Pipfile configuration
    """

    test_arguments: List[str]
    """
    audit snyk test arguments.

    The snyk test arguments

    default:
      - --all-projects
      - --fail-on=all
      - --severity-threshold=medium
    """

    monitor_arguments: List[str]
    """
    audit snyk monitor arguments.

    The snyk monitor arguments

    default:
      - --all-projects
    """

    fix_arguments: List[str]
    """
    audit snyk fix arguments.

    The snyk fix arguments

    default:
      - --all-projects
    """

    fix_github_create_pull_request_arguments: List[str]
    """
    audit snyk fix pull request arguments.

    The snyk fix pull request extra arguments

    default:
      - --fill
      - --label=dependencies
    """

    pip_install_arguments: List[str]
    """
    audit snyk pip install arguments.

    The snyk pip install arguments

    default:
      - --user
    """

    pipenv_sync_arguments: List[str]
    """
    audit snyk pipenv sync arguments.

    The snyk pipenv sync arguments

    default:
      []
    """

    files_no_install: List[str]
    """
    audit snyk files no install.

    The list of files to not install

    default:
      []
    """


AuditWithSnyk = Union["AuditSnykConfig", bool]
"""
Audit with snyk.

The audit snyk configuration

oneOf
"""


CHECKS_DEFAULT = {
    "print_versions": {
        "versions": [
            {"name": "c2cciutils", "cmd": ["c2cciutils", "--version"]},
            {"name": "codespell", "cmd": ["codespell", "--version"], "prefix": "codespell "},
            {"name": "java", "cmd": ["java", "-version"]},
            {"name": "python", "cmd": ["python3", "--version"]},
            {"name": "pip", "cmd": ["python3", "-m", "pip", "--version"]},
            {"name": "node", "prefix": "node ", "cmd": ["node", "--version"]},
            {"name": "npm", "prefix": "npm ", "cmd": ["npm", "--version"]},
            {"name": "docker", "cmd": ["docker", "--version"]},
            {"name": "docker-compose", "cmd": ["docker-compose", "--version"]},
            {"name": "kubectl", "cmd": ["kubectl", "version"]},
            {"name": "make", "cmd": ["make", "--version"], "prefix": "make "},
            {"name": "pip_packages", "cmd": ["pip", "freeze", "--all"], "prefix": "pip packages:\n"},
            {
                "name": "npm_packages",
                "cmd": ["npm", "list", "--all", "--global"],
                "prefix": "npm packages:\n",
            },
        ]
    },
    "print_config": True,
    "print_environment_variables": True,
    "print_github_event": True,
    "gitattribute": True,
    "eof": True,
    "workflows": True,
    "black": True,
    "isort": True,
    "codespell": True,
    "prettier": True,
    "snyk": True,
    "snyk_code": False,
    "snyk_iac": False,
    "snyk_fix": False,
}
"""Default value of the field path 'configuration checks'"""


CHECKS_SNYK_ARGUMENTS_DEFAULT = ["--severity-threshold=medium"]
"""Default value of the field path 'Checks snyk configuration arguments'"""


CHECKS_SNYK_CODE_ARGUMENTS_DEFAULT = ["--all-projects", "--severity-threshold=medium"]
"""Default value of the field path 'Checks snyk code configuration arguments'"""


CHECKS_SNYK_FIX_ARGUMENTS_DEFAULT: List[Any] = []
"""Default value of the field path 'Checks snyk fix configuration arguments'"""


CHECKS_SNYK_IAC_ARGUMENTS_DEFAULT = ["--severity-threshold=medium"]
"""Default value of the field path 'Checks snyk iac configuration arguments'"""


CODESPELL_ARGUMENTS_DEFAULT = ["--quiet-level=2", "--check-filenames", "--ignore-words-list=ro"]
"""Default value of the field path 'Checks codespell config  arguments'"""


CODESPELL_DICTIONARIES_DEFAULT = ["clear", "rare", "informal", "code", "names", "en-GB_to_en-US"]
"""Default value of the field path 'Checks codespell config  internal_dictionaries'"""


CODESPELL_IGNORE_REGULAR_EXPRESSION_DEFAULT = ["(.*/)?poetry\\.lock", "(.*/)?package-lock\\.json"]
"""Default value of the field path 'Checks codespell config  ignore_re'"""


class Checks(TypedDict, total=False):
    """
    Checks.

    The checkers configurations

    default:
      black: true
      codespell: true
      eof: true
      gitattribute: true
      isort: true
      prettier: true
      print_config: true
      print_environment_variables: true
      print_github_event: true
      print_versions:
        versions:
        - cmd:
          - c2cciutils
          - --version
          name: c2cciutils
        - cmd:
          - codespell
          - --version
          name: codespell
          prefix: 'codespell '
        - cmd:
          - java
          - -version
          name: java
        - cmd:
          - python3
          - --version
          name: python
        - cmd:
          - python3
          - -m
          - pip
          - --version
          name: pip
        - cmd:
          - node
          - --version
          name: node
          prefix: 'node '
        - cmd:
          - npm
          - --version
          name: npm
          prefix: 'npm '
        - cmd:
          - docker
          - --version
          name: docker
        - cmd:
          - docker-compose
          - --version
          name: docker-compose
        - cmd:
          - kubectl
          - version
          name: kubectl
        - cmd:
          - make
          - --version
          name: make
          prefix: 'make '
        - cmd:
          - pip
          - freeze
          - --all
          name: pip_packages
          prefix: 'pip packages:
            '
        - cmd:
          - npm
          - list
          - --all
          - --global
          name: npm_packages
          prefix: 'npm packages:
            '
      snyk: true
      snyk_code: false
      snyk_fix: false
      snyk_iac: false
      workflows: true
    """

    black: "ChecksBlack"
    codespell: "ChecksCodespell"
    eof: "ChecksEof"
    gitattribute: "ChecksGitattribute"
    isort: "ChecksIsort"
    print_config: "ChecksPrintConfig"
    workflows: "ChecksWorkflows"
    snyk: "ChecksSnyk"
    snyk_code: "ChecksSnykCode"
    snyk_iac: "ChecksWithSnykIac"
    snyk_fix: "ChecksWithSnykFix"
    prettier: "ChecksPrettier"
    print_versions: "PrintVersions"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """


ChecksBlack = Union["ChecksBlackConfig", bool]
"""
Checks Black.

The Black check configuration

oneOf
"""


class ChecksBlackConfig(TypedDict, total=False):
    """
    Checks black config.

    The Black check configuration
    """

    properties: Dict[str, Any]
    ignore_patterns_re: List[str]
    """
    List of regular expression that should be ignored

    default:
      []
    """


ChecksCodespell = Union["ChecksCodespellConfig", bool]
"""
Checks codespell.

The codespell check configuration

oneOf
"""


class ChecksCodespellConfig(TypedDict, total=False):
    """
    Checks codespell config .

    The codespell check configuration
    """

    internal_dictionaries: List[str]
    """
    codespell dictionaries.

    List of argument that will be added to the codespell command

    default:
      - clear
      - rare
      - informal
      - code
      - names
      - en-GB_to_en-US
    """

    arguments: List[str]
    """
    codespell arguments.

    List of argument that will be added to the codespell command

    default:
      - --quiet-level=2
      - --check-filenames
      - --ignore-words-list=ro
    """

    ignore_re: List[str]
    """
    codespell ignore regular expression.

    List of regular expression that should be ignored

    default:
      - (.*/)?poetry\.lock
      - (.*/)?package-lock\.json
    """


ChecksEof = bool
"""
checks eof.

Check the end-of-file
"""


ChecksGitattribute = bool
"""
checks gitattribute.

Run the Git attributes check
"""


ChecksIsort = Union["ChecksIsortConfig", bool]
"""
checks isort.

The isort check configuration

oneOf
"""


class ChecksIsortConfig(TypedDict, total=False):
    """
    checks isort config.

    The isort check configuration
    """

    ignore_patterns_re: List[str]
    """
    List of regular expression that should be ignored

    default:
      []
    """


ChecksPrettier = Union["ChecksPrettierConfig", bool]
"""
Checks Prettier.

The Prettier check configuration

oneOf
"""


class ChecksPrettierConfig(TypedDict, total=False):
    """
    Checks Prettier config.

    The Prettier check configuration
    """

    properties: Dict[str, Any]


ChecksPrintConfig = bool
"""
Checks print config.

The print the configuration including the auto-generated parts
"""


ChecksSnyk = Union["ChecksSnykConfiguration", bool]
"""
Checks snyk.

The check snyk configuration

oneOf
"""


ChecksSnykCode = Union["ChecksSnykCodeConfiguration", bool]
"""
Checks snyk code.

The check snyk code configuration

oneOf
"""


class ChecksSnykCodeConfiguration(TypedDict, total=False):
    """Checks snyk code configuration."""

    arguments: List[str]
    """
    checks snyk code arguments.

    The snyk code test arguments

    default:
      - --all-projects
      - --severity-threshold=medium
    """


class ChecksSnykConfiguration(TypedDict, total=False):
    """Checks snyk configuration."""

    arguments: List[str]
    """
    checks snyk arguments.

    The snyk code test arguments

    default:
      - --severity-threshold=medium
    """


class ChecksSnykFixConfiguration(TypedDict, total=False):
    """Checks snyk fix configuration."""

    arguments: List[str]
    """
    checks snyk fix arguments.

    The snyk code test arguments

    default:
      []
    """


class ChecksSnykIacConfiguration(TypedDict, total=False):
    """Checks snyk iac configuration."""

    arguments: List[str]
    """
    checks snyk iac arguments.

    The snyk code test arguments

    default:
      - --severity-threshold=medium
    """


ChecksWithSnykFix = Union["ChecksSnykFixConfiguration", bool]
"""
Checks with snyk fix.

The check snyk fix configuration

oneOf
"""


ChecksWithSnykIac = Union["ChecksSnykIacConfiguration", bool]
"""
Checks with snyk iac.

The check snyk iac configuration

oneOf
"""


ChecksWorkflows = bool
"""
checks workflows.

The workflows checks configuration
"""


# configuration.
#
# C2C CI utils configuration file
Configuration = TypedDict(
    "Configuration",
    {
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "audit": "Audit",
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "checks": "Checks",
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "pr-checks": "PullRequestChecks",
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "publish": "Publish",
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "version": "Version",
        # WARNING: The required are not correctly taken in account,
        # See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
        "k8s": "K8SConfiguration",
    },
    total=False,
)


DB_CONFIGURATION_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'K8s configuration db'"""


DOCKER_DISPATCH_EVENT_TYPE_DEFAULT = "image-update"
"""Default value of the field path 'dispatch config event-type'"""


DOCKER_DISPATCH_REPOSITORY_DEFAULT = "camptocamp/argocd-gs-gmf-apps"
"""Default value of the field path 'dispatch config repository'"""


DOCKER_REPOSITORY_DEFAULT = {
    "github": {"server": "ghcr.io", "versions": ["version_tag", "version_branch", "rebuild"]},
    "dockerhub": {},
}
"""Default value of the field path 'Publish Docker config repository'"""


# DB configuration.
#
# Database configuration
#
# default:
#   {}
DbConfiguration = TypedDict(
    "DbConfiguration",
    {
        # K8S DB chart options.
        #
        # default:
        #   auth.postgresPassword: mySuperTestingPassword
        #   persistence.enabled: 'false'
        #   tls.autoGenerated: 'true'
        #   tls.enabled: 'true'
        #   volumePermissions.enabled: 'true'
        "chart-options": Dict[str, str],
    },
    total=False,
)


# dispatch config.
#
# Send a dispatch event to an other repository
DispatchConfig = TypedDict(
    "DispatchConfig",
    {
        # Docker dispatch repository.
        #
        # The repository name to be triggered
        #
        # default: camptocamp/argocd-gs-gmf-apps
        "repository": str,
        # Docker dispatch event type.
        #
        # The event type to be triggered
        #
        # default: image-update
        "event-type": str,
    },
    total=False,
)


# K3d configuration.
#
# default:
#   {}
K3DConfiguration = TypedDict(
    "K3DConfiguration",
    {
        # K3D install commands.
        #
        # default:
        #   - - k3d
        #     - cluster
        #     - create
        #     - test-cluster
        #     - --no-lb
        #     - --no-rollback
        "install-commands": List[List[str]],
    },
    total=False,
)


K3D_CONFIGURATION_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'K8s configuration k3d'"""


K3D_INSTALL_COMMANDS_DEFAULT = [["k3d", "cluster", "create", "test-cluster", "--no-lb", "--no-rollback"]]
"""Default value of the field path 'K3d configuration install-commands'"""


class K8SConfiguration(TypedDict, total=False):
    """
    K8s configuration.

    default:
      {}
    """

    k3d: "K3DConfiguration"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """

    db: "DbConfiguration"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """


K8S_CONFIGURATION_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'configuration k8s'"""


K8S_DB_CHART_OPTIONS_DEFAULT = {
    "persistence.enabled": "false",
    "tls.enabled": "true",
    "tls.autoGenerated": "true",
    "auth.postgresPassword": "mySuperTestingPassword",
    "volumePermissions.enabled": "true",
}
"""Default value of the field path 'DB configuration chart-options'"""


PUBLISH_DEFAULT = {
    "print_versions": {
        "versions": [
            {"name": "c2cciutils", "cmd": ["c2cciutils", "--version"]},
            {"name": "python", "cmd": ["python3", "--version"]},
            {"name": "twine", "cmd": ["twine", "--version"]},
            {"name": "docker", "cmd": ["docker", "--version"]},
        ]
    },
    "pypi": {"versions": ["version_tag"], "packages": "<auto-detected>"},
    "docker": {"images": "<auto-detected>"},
    "helm": {"versions": ["version_tag"], "folders": "<auto-detected>"},
}
"""Default value of the field path 'configuration publish'"""


PUBLISH_DOCKER_IMAGE_GROUP_DEFAULT = "default"
"""Default value of the field path 'Publish Docker image group'"""


PUBLISH_DOCKER_IMAGE_TAGS_DEFAULT = ["{version}"]
"""Default value of the field path 'Publish Docker image tags'"""


PUBLISH_DOCKER_LATEST_DEFAULT = True
"""Default value of the field path 'Publish Docker config latest'"""


PUBLISH_DOCKER_REPOSITORY_VERSIONS_DEFAULT = ["version_tag", "version_branch", "rebuild", "feature_branch"]
"""Default value of the field path 'Publish Docker repository versions'"""


PUBLISH_DOCKER_SNYK_MONITOR_ARGS_DEFAULT = ["--app-vulns"]
"""Default value of the field path 'Publish Docker config snyk monitor_args'"""


PUBLISH_DOCKER_SNYK_TEST_ARGS_DEFAULT = ["--app-vulns", "--severity-threshold=critical"]
"""Default value of the field path 'Publish Docker config snyk test_args'"""


PUBLISH_GOOGLE_CALENDAR_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'Publish google_calendar'"""


PUBLISH_GOOGLE_CALENDAR_ON_DEFAULT = ["version_branch", "version_tag", "rebuild"]
"""Default value of the field path 'Publish Google calendar config on'"""


PUBLISH_PIP_PACKAGE_GROUP_DEFAULT = "default"
"""Default value of the field path 'publish pypi package group'"""


PUBLISH_PYPI_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'Publish pypi'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_FIRST_CAPITAL_DEFAULT = True
"""Default value of the field path 'pull request checks commits messages configuration check_first_capital'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_FIXUP_DEFAULT = True
"""Default value of the field path 'pull request checks commits messages configuration check_fixup'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_MIN_HEAD_LENGTH_DEFAULT = 5
"""Default value of the field path 'pull request checks commits messages configuration min_head_length'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_NO_MERGE_COMMITS_DEFAULT = True
"""Default value of the field path 'pull request checks commits messages configuration check_no_merge_commits'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_NO_OWN_REVERT_DEFAULT = True
"""Default value of the field path 'pull request checks commits messages configuration check_no_own_revert'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_ONLY_HEAD_DEFAULT = True
"""Default value of the field path 'pull request checks commits spelling configuration only_head'"""


PULL_REQUEST_CHECKS_COMMITS_MESSAGES_SQUASH_DEFAULT = True
"""Default value of the field path 'pull request checks commits messages configuration check_squash'"""


PULL_REQUEST_CHECKS_DEFAULT = {
    "commits_messages": True,
    "commits_spell": True,
    "pull_request_spell": True,
    "pull_request_labels": True,
    "add_issue_link": True,
}
"""Default value of the field path 'configuration pr-checks'"""


PULL_REQUEST_CHECKS_ONLY_HEAD_DEFAULT = True
"""Default value of the field path 'pull request checks pull request spelling configuration only_head'"""


class PrintVersions(TypedDict, total=False):
    """
    Print versions.

    The print versions configuration
    """

    versions: List["_PrintVersionsVersionsItem"]


class Publish(TypedDict, total=False):
    """
    Publish.

    The publishing configurations

    default:
      docker:
        images: <auto-detected>
      helm:
        folders: <auto-detected>
        versions:
        - version_tag
      print_versions:
        versions:
        - cmd:
          - c2cciutils
          - --version
          name: c2cciutils
        - cmd:
          - python3
          - --version
          name: python
        - cmd:
          - twine
          - --version
          name: twine
        - cmd:
          - docker
          - --version
          name: docker
      pypi:
        packages: <auto-detected>
        versions:
        - version_tag
    """

    docker: "PublishDocker"
    pypi: "PublishPypi"
    helm: "PublishHelm"
    google_calendar: "PublishGoogleCalendar"
    print_versions: "PrintVersions"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """


PublishDocker = Union["PublishDockerConfig", Literal[False]]
"""
Publish Docker.

The configuration used to publish on Docker

oneOf
"""


class PublishDockerConfig(TypedDict, total=False):
    """
    Publish Docker config.

    The configuration used to publish on Docker
    """

    latest: bool
    """
    Publish Docker latest.

    Publish the latest version on tag latest

    default: True
    """

    images: List["PublishDockerImage"]
    """List of images to be published"""

    repository: Dict[str, "PublishDockerRepository"]
    """
    Docker repository.

    The repository where we should publish the images

    default:
      dockerhub: {}
      github:
        server: ghcr.io
        versions:
        - version_tag
        - version_branch
        - rebuild
    """

    dispatch: Union["DispatchConfig", Literal[False]]
    """
    Send a dispatch event to an other repository

    default:
      {}

    oneOf
    """

    snyk: "_PublishDockerConfigSnyk"
    """
    WARNING: The required are not correctly taken in account,
    See: https://github.com/camptocamp/jsonschema-gentypes/issues/6
    """


class PublishDockerImage(TypedDict, total=False):
    """Publish Docker image."""

    group: str
    """
    Publish Docker image group.

    The image is in the group, should be used with the --group option of c2cciutils-publish script

    default: default
    """

    name: str
    """The image name"""

    tags: List[str]
    """
    publish docker image tags.

    The tag name, will be formatted with the version=<the version>, the image with version=latest should be present when we call the c2cciutils-publish script

    default:
      - '{version}'
    """


class PublishDockerRepository(TypedDict, total=False):
    """Publish Docker repository."""

    server: str
    """The server URL"""

    versions: List[str]
    """
    Publish Docker repository versions.

    The kind or version that should be published, tag, branch or value of the --version argument of the c2cciutils-publish script

    default:
      - version_tag
      - version_branch
      - rebuild
      - feature_branch
    """


PublishGoogleCalendar = Union["PublishGoogleCalendarConfig", Literal[False]]
"""
Publish Google calendar.

The configuration to publish on Google Calendar

default:
  {}

oneOf
"""


class PublishGoogleCalendarConfig(TypedDict, total=False):
    """
    Publish Google calendar config.

    The configuration to publish on Google Calendar
    """

    on: List[str]
    """
    Publish Google calendar on.

    default:
      - version_branch
      - version_tag
      - rebuild
    """


PublishHelm = Union["PublishHelmConfig", Literal[False]]
"""
publish helm.

Configuration to publish Helm charts on GitHub release

oneOf
"""


class PublishHelmConfig(TypedDict, total=False):
    """
    publish helm config.

    Configuration to publish on Helm charts on GitHub release
    """

    folders: List[str]
    """The folders that will be published"""

    versions: List[str]
    """The kind or version that should be published, tag, branch or value of the --version argument of the c2cciutils-publish script"""


PublishPypi = Union["PublishPypiConfig", Literal[False]]
"""
publish pypi.

Configuration to publish on pypi

default:
  {}

oneOf
"""


class PublishPypiConfig(TypedDict, total=False):
    """
    publish pypi config.

    Configuration to publish on pypi
    """

    packages: List["PublishPypiPackage"]
    """The configuration of packages that will be published"""

    versions: List[str]
    """The kind or version that should be published, tag, branch or value of the --version argument of the c2cciutils-publish script"""


class PublishPypiPackage(TypedDict, total=False):
    """
    publish pypi package.

    The configuration of package that will be published
    """

    group: str
    """
    Publish pip package group.

    The image is in the group, should be used with the --group option of c2cciutils-publish script

    default: default
    """

    path: str
    """The path of the pypi package"""

    build_command: List[str]
    """The command used to do the build"""


class PullRequestChecks(TypedDict, total=False):
    """
    Pull request checks.

    The PR check configuration

    default:
      add_issue_link: true
      commits_messages: true
      commits_spell: true
      pull_request_labels: true
      pull_request_spell: true
    """

    print_event: "PullRequestChecksPrintEvent"
    commits_messages: "PullRequestChecksCommitsMessages"
    commits_spell: "PullRequestChecksCommitsSpelling"
    pull_request_spell: "PullRequestChecksPullRequestSpelling"
    pull_request_labels: "PullRequestChecksRequestLabels"
    add_issue_link: "PullRequestChecksAddIssueLink"


PullRequestChecksAddIssueLink = bool
"""pull request checks add issue link."""


PullRequestChecksCommitsMessages = Union["PullRequestChecksCommitsMessagesConfiguration", bool]
"""
pull request checks commits messages.

Check the pull request commits messages

oneOf
"""


class PullRequestChecksCommitsMessagesConfiguration(TypedDict, total=False):
    """
    pull request checks commits messages configuration.

    The commit message check configuration
    """

    check_fixup: bool
    """
    pull request checks commits messages fixup.

    Check that we don't have one fixup commit in the pull request

    default: True
    """

    check_squash: bool
    """
    pull request checks commits messages squash.

    Check that we don't have one squash commit in the pull request

    default: True
    """

    check_first_capital: bool
    """
    pull request checks commits messages first capital.

    Check that the all the commits message starts with a capital letter

    default: True
    """

    min_head_length: int
    """
    pull request checks commits messages min head length.

    Check that the commits message head is at least this long, use 0 to disable

    default: 5
    """

    check_no_merge_commits: bool
    """
    pull request checks commits messages no merge commits.

    Check that we don't have merge commits in the pull request

    default: True
    """

    check_no_own_revert: bool
    """
    pull request checks commits messages no own revert.

    Check that we don't have reverted one of our commits in the pull request

    default: True
    """


PullRequestChecksCommitsSpelling = Union["PullRequestChecksCommitsSpellingConfiguration", bool]
"""
pull request checks commits spelling.

oneOf
"""


class PullRequestChecksCommitsSpellingConfiguration(TypedDict, total=False):
    """
    pull request checks commits spelling configuration.

    Configuration used to check the spelling of the commits
    """

    only_head: bool
    """
    pull request checks commits messages only head.

    default: True
    """


PullRequestChecksPrintEvent = bool
"""
pull request checks print event.

Print the GitHub event object
"""


PullRequestChecksPullRequestSpelling = Union["PullRequestChecksPullRequestSpellingConfiguration", bool]
"""
pull request checks pull request spelling.

oneOf
"""


class PullRequestChecksPullRequestSpellingConfiguration(TypedDict, total=False):
    """
    pull request checks pull request spelling configuration.

    Configuration used to check the spelling of the title and body of the pull request
    """

    only_head: bool
    """
    pull request checks only head.

    default: True
    """


PullRequestChecksRequestLabels = bool
"""
pull request checks request labels.

According the create changelog configuration
"""


class Version(TypedDict, total=False):
    """
    Version.

    The version configurations
    """

    branch_to_version_re: "VersionTransform"
    tag_to_version_re: "VersionTransform"


VersionTransform = List["_VersionTransformItem"]
"""
Version transform.

A version transformer definition
"""


_CHECKS_BLACK_CONFIG_IGNORE_PATTERNS_RE_DEFAULT: List[Any] = []
"""Default value of the field path 'Checks black config ignore_patterns_re'"""


_CHECKS_ISORT_CONFIG_IGNORE_PATTERNS_RE_DEFAULT: List[Any] = []
"""Default value of the field path 'checks isort config ignore_patterns_re'"""


_PUBLISH_DOCKER_CONFIG_DISPATCH_DEFAULT: Dict[str, Any] = {}
"""Default value of the field path 'Publish Docker config dispatch'"""


class _PrintVersionsVersionsItem(TypedDict, total=False):
    cmd: List[str]
    """The command that should be used"""

    name: str
    """The name"""

    prefix: str
    """Prefix added when we print the version"""


class _PublishDockerConfigSnyk(TypedDict, total=False):
    """Checks the published images with Snyk"""

    monitor_args: Union[List[str], Literal[False]]
    """
    Publish docker snyk monitor args.

    The arguments to pass to the Snyk container monitor command

    default:
      - --app-vulns

    oneOf
    """

    test_args: Union[List[str], Literal[False]]
    """
    Publish docker snyk test args.

    The arguments to pass to the Snyk container test command

    default:
      - --app-vulns
      - --severity-threshold=critical

    oneOf
    """


_VersionTransformItem = TypedDict(
    "_VersionTransformItem",
    {
        # The from regular expression
        "from": str,
        # The expand regular expression: https://docs.python.org/3/library/re.html#re.Match.expand
        "to": str,
    },
    total=False,
)



# -*- coding: utf-8 -*-

"""
Load prettier.js and improve the API.
"""

import difflib
import os
from types import TracebackType
from typing import Any, Dict, Optional, Type, cast

import node_vm2
import yaml


class PrettierModule:
    """
    Expose and improve the prettier.js API.
    """

    def __init__(self, module: node_vm2.NodeVMModule) -> None:
        """
        Initialize.
        """
        self.module = module

    def get_info(self, filename: str) -> Dict[str, Any]:
        """
        Get the Prettier information related to a file.

        Arguments:
            filename: The file name to consider
        """
        return cast(Dict[str, Any], self.module.call_member("getFileInfo", filename))

    def check(self, filename: str, config: Dict[str, Any]) -> bool:
        """
        Check a file.

        Arguments:
            filename: The file name to check
            config: The Prettier config related to the file
        """
        try:
            with open(filename, encoding="utf-8") as the_file_to_check:
                data = the_file_to_check.read()
                success = cast(bool, self.module.call_member("check", data, config))
            if not success:
                new_data = self.module.call_member("format", data, config)
                print()
                printable_diff = "".join(
                    difflib.unified_diff(
                        data.splitlines(True),
                        new_data.splitlines(True),
                        filename,
                        filename + "-formated",
                    )
                )
                print(f"Wrong file formatting with config:\n{self.dump_yaml(config)}{printable_diff}")
            return success
        except node_vm2.VMError as exception:
            print(
                f"ERROR on check the file '{filename}' with config:\n"
                f"{self.dump_yaml(config)}\n{exception}"
            )
            return False

    def dump_yaml(self, data: Dict[str, Any], filename: str = "ci/config.yaml") -> str:
        """
        Format some YAML.

        Arguments:
            data: The YAML to be dump and format
            filename: The file name used to get the Prettier config
        """
        return self.format_str(yaml.dump(data, default_flow_style=False, Dumper=yaml.SafeDumper), filename)

    def format_str(self, data: str, filename: str = "ci/config.yaml") -> str:
        """
        Format some data.

        Arguments:
            data: The data to be formatted
            filename: The file name used to get the Prettier config
        """
        info = self.get_info(filename)
        if info.get("info", {}).get("ignored", False):
            return data
        if not info.get("info", {}).get("inferredParser"):
            return data
        config = info["config"]
        config["parser"] = info["info"]["inferredParser"]
        try:
            return cast(str, self.module.call_member("format", data, config))
        except node_vm2.VMError as exception:
            print(exception)
            return data

    def format(self, filename: str, config: Dict[str, Any]) -> bool:
        """
        Format a file.

        Arguments:
            filename: The file name to format
            config: The Prettier config related to the file
        """
        try:
            with open(filename, encoding="utf-8") as the_file_to_format:
                new_data = self.module.call_member("format", the_file_to_format.read(), config)
            with open(filename, "w", encoding="utf-8") as the_file_to_format:
                the_file_to_format.write(new_data)
        except node_vm2.VMError as exception:
            print(
                f"ERROR on formatted the file '{filename}' with config:\n"
                f"{self.dump_yaml(config)}\n{exception}"
            )
            return False
        return True


class Prettier:
    """
    Load prettier.js.
    """

    module: Optional[PrettierModule] = None

    def __enter__(self) -> PrettierModule:
        """
        Initialize.
        """
        with open(os.path.join(os.path.dirname(__file__), "prettier.js"), encoding="utf-8") as query_open:
            javascript = query_open.read()

        self.module = PrettierModule(
            node_vm2.NodeVM.code(
                javascript,
                os.path.join(os.path.dirname(__file__), "node_modules"),
                console="inherit",
                require={"external": True},
            )
        )
        return self.module

    def __exit__(
        self,
        tpe: Optional[Type[BaseException]],
        value: Optional[BaseException],
        traceback: Optional[TracebackType],
    ) -> Any:
        """
        Close.
        """
        assert self.module is not None

        return self.module.module.__exit__(tpe, value, traceback)


def dump_yaml(data: Dict[str, Any], filename: str = "ci/config.yaml") -> str:
    """
    Dump the YAML according to the prettier configuration.

    Arguments:
        data: The content of the YAML
        filename: The destination file name
    """
    with Prettier() as prettier:
        return prettier.dump_yaml(data, filename)



"""
The publishing functions.
"""

import argparse
import datetime
import glob
import os
import pickle  # nosec
import re
import subprocess  # nosec
import sys
import uuid
from typing import List, Optional

import ruamel.yaml
import tomlkit
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build

import c2cciutils.configuration


class GoogleCalendar:
    """
    Interact with the Google Calendar API.
    """

    # pylint: disable=too-many-instance-attributes
    def __init__(self) -> None:
        """
        Initialize.
        """
        self.scopes = ["https://www.googleapis.com/auth/calendar"]  # in fact it is better to hard-code this
        self.credentials_pickle_file = os.environ.get("TMP_CREDS_FILE", f"/tmp/{uuid.uuid4()}.pickle")
        self.credentials_json_file = os.environ.get(
            "GOOGLE_CREDS_JSON_FILE", "~/google-credentials-c2cibot.json"
        )  # used to refresh the refresh_token or to initialize the credentials the first time
        self.calendar_id = os.environ.get(
            "GOOGLE_CALENDAR_ID", c2cciutils.gopass("gs/ci/google_calendar/calendarId")
        )
        self.token = os.environ.get("GOOGLE_TOKEN", c2cciutils.gopass("gs/ci/google_calendar/token"))
        self.token_uri = os.environ.get(
            "GOOGLE_TOKEN_URI", c2cciutils.gopass("gs/ci/google_calendar/token_uri")
        )
        self.refresh_token = os.environ.get(
            "GOOGLE_REFRESH_TOKEN",
            c2cciutils.gopass("gs/ci/google_calendar/refresh_token"),
        )
        self.client_id = os.environ.get(
            "GOOGLE_CLIENT_ID", c2cciutils.gopass("gs/ci/google_calendar/client_id")
        )
        self.client_secret = os.environ.get(
            "GOOGLE_CLIENT_SECRET",
            c2cciutils.gopass("gs/ci/google_calendar/client_secret"),
        )

        self.creds: Credentials = self.init_calendar_service()
        self._update_creds()
        self.service = build("calendar", "v3", credentials=self.creds)

    def init_calendar_service(self) -> Credentials:
        """
        Initialize the calendar service.
        """
        # The file token pickle stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if os.path.exists(self.credentials_pickle_file):
            with open(self.credentials_pickle_file, "rb") as token:
                creds = pickle.load(token)  # nosec
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                if self.token:
                    creds = Credentials(
                        self.token,
                        refresh_token=self.refresh_token,
                        token_uri=self.token_uri,
                        client_id=self.client_id,
                        client_secret=self.client_secret,
                        scopes=self.scopes,
                    )
                else:
                    flow = InstalledAppFlow.from_client_secrets_file(self.credentials_json_file, self.scopes)
                    creds = flow.run_local_server(port=0)
                    self.refresh_token = creds

            # Save the credentials for the next run
            with open(self.credentials_pickle_file, "wb") as token:
                pickle.dump(creds, token)

    def _update_creds(self) -> None:
        """
        Update the credentials.
        """
        self.client_id = self.creds.client_id
        self.client_secret = self.creds.client_secret
        self.token = self.creds.token
        self.token_uri = self.creds.token_uri
        self.refresh_token = self.creds.refresh_token

    def print_all_calendars(self) -> None:
        """
        Print all calendar events.
        """
        # list all the calendars that the user has access to.
        # used to debug credentials
        print("Getting list of calendars")
        calendars_result = self.service.calendarList().list().execute()

        calendars = calendars_result.get("items", [])

        if not calendars:
            print("::error::No calendars found.")
        for calendar in calendars:
            summary = calendar["summary"]
            event_id = calendar["id"]
            primary = "Primary" if calendar.get("primary") else ""
            print(f"{summary}\t{event_id}\t{primary}")

    def print_latest_events(self, time_min: Optional[datetime.datetime] = None) -> None:
        """
        Print latest events.

        Arguments:
            time_min: The time to be considered.
        """
        now = datetime.datetime.utcnow()
        if not time_min:
            time_min = datetime.datetime.utcnow() - datetime.timedelta(days=30)
        events_result = (
            self.service.events()
            .list(
                calendarId=self.calendar_id,
                timeMin=time_min.isoformat() + "Z",
                timeMax=now.isoformat() + "Z",
                singleEvents=True,
                orderBy="startTime",
            )
            .execute()
        )
        events = events_result.get("items", [])

        if not events:
            print("::error::No upcoming events found.")
        for event in events:
            start = event["start"].get("dateTime", event["start"].get("date"))
            print(start, event["summary"])

    def create_event(
        self,
        summary: str = f"dummy/image:{datetime.datetime.now().isoformat()}",
        description: str = "description",
    ) -> None:
        """
        Create a calendar event.

        Arguments:
            summary: The event summary
            description: The event description
        """
        now = datetime.datetime.now()
        start = now.isoformat()
        end = (now + datetime.timedelta(minutes=15)).isoformat()
        body = {
            "summary": summary,
            "description": description,
            "start": {"dateTime": start, "timeZone": "Europe/Zurich"},
            "end": {"dateTime": end, "timeZone": "Europe/Zurich"},
        }

        event_result = self.service.events().insert(calendarId=self.calendar_id, body=body).execute()
        print(f"Created event with id: {event_result['id']}")

    def _print_credentials(self) -> None:
        """
        Print the credentials.
        """
        # UNSAFE: DO NEVER PRINT CREDENTIALS IN CI ENVIRONMENT, DEBUG ONLY!!!!
        print(self.creds.to_json())

    def save_credentials_to_gopass(self) -> None:
        """
        Save the calendar credentials to gopass.
        """
        objects_to_save = {
            "gs/ci/google_calendar/calendarId": self.calendar_id,
            "gs/ci/google_calendar/token": self.token,
            "gs/ci/google_calendar/token_uri": self.token_uri,
            "gs/ci/google_calendar/refresh_token": self.refresh_token,
            "gs/ci/google_calendar/client_id": self.client_id,
            "gs/ci/google_calendar/client_secret": self.client_secret,
        }
        for key, secret in objects_to_save.items():
            assert secret is not None
            c2cciutils.gopass_put(secret, key)

    def __del__(self) -> None:
        if os.path.exists(self.credentials_pickle_file):
            os.remove(self.credentials_pickle_file)


def main_calendar() -> None:
    """
    Run the calendar main function.
    """
    parser = argparse.ArgumentParser(
        description="Interact with google API for the Docker publishing calendar"
    )
    parser.add_argument(
        "--refresh-gopass-credentials",
        action="store_true",
        help="Refresh the credentials in gopass using google API",
    )
    parser.add_argument(
        "--show-events-since",
        help="show the calendar events since a date in 'YYYY-mm-dd' format",
        type=lambda s: datetime.datetime.strptime(s, "%Y-%m-%d"),
    )
    parser.add_argument(
        "--create-test-event",
        action="store_true",
        help="Create a dummy event to check that the calendar settings are correct",
    )
    args = parser.parse_args()

    if args.show_events_since or args.refresh_gopass_credentials or args.create_test_event:
        google_calendar = GoogleCalendar()
    else:
        parser.print_help()

    if args.show_events_since:
        google_calendar.print_latest_events(args.show_events_since)

    if args.refresh_gopass_credentials:
        google_calendar.save_credentials_to_gopass()

    if args.create_test_event:
        google_calendar.create_event()


def pip(
    package: c2cciutils.configuration.PublishPypiPackage, version: str, version_type: str, publish: bool
) -> bool:
    """
    Publish to pypi.

    Arguments:
        version: The version that will be published
        version_type: Describe the kind of release we do: rebuild (specified using --type), version_tag,
                    version_branch, feature_branch, feature_tag (for pull request)
        publish: If False only check the package
        package: The package configuration
    """

    print(f"::group::{'Publishing' if publish else 'Checking'} '{package.get('path')}' to pypi")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        env = {}
        env["VERSION"] = version
        env["VERSION_TYPE"] = version_type
        full_repo = c2cciutils.get_repository()
        full_repo_split = full_repo.split("/")
        master_branch, _ = c2cciutils.get_master_branch(full_repo_split)
        is_master = master_branch == version
        env["IS_MASTER"] = "TRUE" if is_master else "FALSE"

        cwd = os.path.abspath(package.get("path", "."))

        dist = os.path.join(cwd, "dist")
        if not os.path.exists(dist):
            os.mkdir(dist)
        if os.path.exists(os.path.join(cwd, "setup.py")):
            cmd = ["python3", "./setup.py", "egg_info", "--no-date"]
            cmd += (
                ["--tag-build=dev" + datetime.datetime.now().strftime("%Y%m%d%H%M%S")]
                if version_type in ("version_branch", "rebuild")
                else []
            )
            cmd.append("bdist_wheel")
        else:
            if not os.path.exists(dist):
                os.mkdir(dist)
            cmd = ["pip", "wheel", "--no-deps", "--wheel-dir=dist", "."]
            if os.path.exists(os.path.join(cwd, "pyproject.toml")):
                use_poetry = False
                if "build_command" not in package:
                    with open(os.path.join(cwd, "pyproject.toml"), encoding="utf-8") as project_file:
                        pyproject = tomlkit.load(project_file)
                    re_splitter = re.compile(r"[<>=]+")
                    for requirement in pyproject.get("build-system", {}).get("requires", []):
                        requirement_split = re_splitter.split(requirement)
                        if requirement_split[0] in ("poetry", "poetry-core"):
                            use_poetry = True
                            break
                    subprocess.run(
                        [
                            "pip",
                            "install",
                            "importlib-metadata<8.0.0",
                            *pyproject.get("build-system", {}).get("requires", []),
                        ],
                        check=True,
                    )
                if use_poetry:
                    freeze = subprocess.run(["pip", "freeze"], check=True, stdout=subprocess.PIPE)
                    for freeze_line in freeze.stdout.decode("utf-8").split("\n"):
                        if freeze_line.startswith("poetry-") or freeze_line.startswith("poetry="):
                            print(freeze_line)
                    env_bash = " ".join([f"{key}={value}" for key, value in env.items()])
                    print(f"Run in {cwd}: {env_bash} poetry build")
                    sys.stdout.flush()
                    sys.stderr.flush()
                    subprocess.run(["poetry", "build"], cwd=cwd, env={**os.environ, **env}, check=True)
                    cmd = []
        if cmd:
            cmd = package.get("build_command", cmd)
            subprocess.check_call(cmd, cwd=cwd, env=env)
        cmd = ["twine"]
        cmd += ["upload", "--verbose", "--disable-progress-bar"] if publish else ["check"]
        cmd += glob.glob(os.path.join(cwd, "dist/*.whl"))
        cmd += glob.glob(os.path.join(cwd, "dist/*.tar.gz"))
        subprocess.check_call(cmd)
        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True


def docker(
    config: c2cciutils.configuration.PublishDockerRepository,
    name: str,
    image_config: c2cciutils.configuration.PublishDockerImage,
    tag_src: str,
    tag_dst: str,
    latest: bool,
    images_full: List[str],
) -> bool:
    """
    Publish to a Docker registry.

    config is like:
        server: # The server fqdn

    image_config is like:
        name: # The image name

    Arguments:
        config: The publishing config
        name: The repository name, just used to print messages
        image_config: The image config
        tag_src: The source tag (usually latest)
        tag_dst: The tag used for publication
        latest: Publish also the tag latest
        images_full: The list of published images (with tag), used to build the dispatch event
    """

    print(f"::group::Publishing {image_config['name']}:{tag_dst} to {name}")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        new_images_full = []
        if "server" in config:
            subprocess.run(
                [
                    "docker",
                    "tag",
                    f"{image_config['name']}:{tag_src}",
                    f"{config['server']}/{image_config['name']}:{tag_dst}",
                ],
                check=True,
            )
            new_images_full.append(f"{config['server']}/{image_config['name']}:{tag_dst}")
            if latest:
                subprocess.run(
                    [
                        "docker",
                        "tag",
                        f"{image_config['name']}:{tag_src}",
                        f"{config['server']}/{image_config['name']}:{tag_src}",
                    ],
                    check=True,
                )
                new_images_full.append(f"{config['server']}/{image_config['name']}:{tag_src}")
        else:
            if tag_src != tag_dst:
                subprocess.run(
                    [
                        "docker",
                        "tag",
                        f"{image_config['name']}:{tag_src}",
                        f"{image_config['name']}:{tag_dst}",
                    ],
                    check=True,
                )
            new_images_full.append(f"{image_config['name']}:{tag_dst}")
            if latest and tag_src != tag_dst:
                new_images_full.append(f"{image_config['name']}:{tag_src}")

        for image in new_images_full:
            subprocess.run(["docker", "push", image], check=True)
        images_full += new_images_full

        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True


def helm(folder: str, version: str, owner: str, repo: str, commit_sha: str, token: str) -> bool:
    """
    Publish to pypi.

    Arguments:
        folder: The folder to be published
        version: The version that will be published
        owner: The GitHub repository owner
        repo: The GitHub repository name
        commit_sha: The sha of the current commit
        token: The GitHub token
    """

    print(f"::group::Publishing Helm chart from '{folder}' to GitHub release")
    sys.stdout.flush()
    sys.stderr.flush()

    try:
        yaml_ = ruamel.yaml.YAML()
        with open(os.path.join(folder, "Chart.yaml"), encoding="utf-8") as open_file:
            chart = yaml_.load(open_file)
        chart["version"] = version
        with open(os.path.join(folder, "Chart.yaml"), "w", encoding="utf-8") as open_file:
            yaml_.dump(chart, open_file)
        for index, dependency in enumerate(chart.get("dependencies", [])):
            if dependency["repository"].startswith("https://"):
                subprocess.run(["helm", "repo", "add", str(index), dependency["repository"]], check=True)

        subprocess.run(["cr", "package", folder], check=True)
        subprocess.run(
            [
                "cr",
                "upload",
                f"--owner={owner}",
                f"--git-repo={repo}",
                f"--commit={commit_sha}",
                "--release-name-template={{ .Version }}",
                f"--token={token}",
            ],
            check=True,
        )
        if not os.path.exists(".cr-index"):
            os.mkdir(".cr-index")
        subprocess.run(
            [
                "cr",
                "index",
                f"--owner={owner}",
                f"--git-repo={repo}",
                f"--charts-repo=https://{owner}.github.io/{repo}",
                "--push",
                "--release-name-template={{ .Version }}",
                f"--token={token}",
            ],
            check=True,
        )
        print("::endgroup::")
    except subprocess.CalledProcessError as exception:
        print(f"Error: {exception}")
        print("::endgroup::")
        print("::error::With error")
        return False
    return True



#!/usr/bin/env python3

import argparse
import os
import subprocess  # nosec
import sys
from typing import Dict, cast

import yaml

import c2cciutils
import c2cciutils.configuration


def _print(message: str) -> None:
    print(message)
    sys.stdout.flush()


def main() -> None:
    """Create and cleanup a test database."""
    parser = argparse.ArgumentParser(
        description="Create and cleanup a test database.",
        formatter_class=argparse.RawTextHelpFormatter,
        epilog="""Database credentials:
    host: test-pg-postgresql
    port: 5432
    user: postgres
    password: mySuperTestingPassword
    database name: postgres""",
    )
    parser.add_argument("--script", help="The script used to initialize the database")
    parser.add_argument("--cleanup", action="store_true", help="Drop the database")

    config = c2cciutils.get_config()
    with open(
        os.path.join(
            os.path.dirname(os.path.dirname(os.path.dirname(__file__))), "applications-versions.yaml"
        ),
        "r",
        encoding="utf-8",
    ) as config_file:
        versions = cast(Dict[str, str], yaml.load(config_file, Loader=yaml.SafeLoader))

    args = parser.parse_args()

    if args.cleanup:
        _print("::group::Cleanup the database")
        subprocess.run([os.environ.get("HELM", "helm"), "uninstall", "test-pg"], check=False)
        _print("::endgroup::")
        sys.exit(0)

    _print("::group::Add repo")
    subprocess.run(
        [os.environ.get("HELM", "helm"), "repo", "add", "bitnami", "https://charts.bitnami.com/bitnami"],
        check=True,
    )
    _print("::endgroup::")

    _print("::group::Install chart")
    subprocess.run(
        [
            os.environ.get("HELM", "helm"),
            "install",
            "test-pg",
            f"--version={versions['postgresql']}",
        ]
        + [
            f"--set={k}={v}"
            for k, v in config.get("k8s", {})
            .get("db", {})
            .get("chart-options", c2cciutils.configuration.K8S_DB_CHART_OPTIONS_DEFAULT)
            .items()
        ]
        + ["bitnami/postgresql"],
        check=True,
    )
    _print("::endgroup::")

    _print("::group::Wait ready")
    subprocess.run(["c2cciutils-k8s-wait", "--selector=app.kubernetes.io/name=postgresql"], check=True)
    _print("::endgroup::")

    if args.script:
        _print("::group::Add data")
        with open(args.script, encoding="utf-8") as script:
            subprocess.run(
                [
                    "kubectl",
                    "run",
                    "test-pg-postgresql-client",
                    "--rm",
                    "--restart=Never",
                    "--namespace=default",
                    "--image=docker.io/bitnami/postgresql",
                    "--env=PGPASSWORD=mySuperTestingPassword",
                    "--stdin=true",
                    "--command",
                    "--",
                    "psql",
                    "--host=test-pg-postgresql",
                    "--username=postgres",
                    "--dbname=postgres",
                    "--port=5432",
                ],
                check=True,
                stdin=script,
            )
        _print("::endgroup::")


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

"""
The checker main function.
"""

import argparse
import sys
import traceback

import c2cciutils.checks


def main() -> None:
    """
    Run the checks.
    """
    parser = argparse.ArgumentParser(description="Run the checks of c2cciutils.")
    parser.add_argument("--fix", action="store_true", help="fix black and isort issues")
    parser.add_argument("--stop", action="store_true", help="stop on first error")
    parser.add_argument("--check", help="runs only the specified check")

    args = parser.parse_args()

    full_config = c2cciutils.get_config()
    config = full_config.get("checks", {})
    success = True
    for key, conf in config.items():
        if conf is not False and (args.check is None or args.check == key):
            check = getattr(c2cciutils.checks, key)
            print(f"::group::Run check {key}")
            try:
                if not check({} if conf is True else conf, full_config, args):
                    success = False
                    print("::endgroup::")
                    if args.stop:
                        sys.exit(1)
                    print("::error::With error")
                    if key in ("black", "isort", "prettier", "codespell"):
                        print("Can be fixed with:")
                        print("python3 -m pip install --requirement=ci/requirements.txt")
                        print(f"c2cciutils-checks --fix --check={key}")
                    if key in ("black", "isort", "prettier"):
                        print("See also documentation for IDE: https://github.com/camptocamp/c2cciutils#ide")
                else:
                    print("::endgroup::")
            except Exception:  # pylint: disable=broad-except
                traceback.print_exc()
                success = False
                print("::endgroup::")
                if args.stop:
                    sys.exit(1)
                print("::error::With error")
    if not success:
        sys.exit(1)


if __name__ == "__main__":
    main()



# MODIFIED - Replace/Update with care

import os
import warnings
import re

from .requirement import Requirement

def parse(req_str_or_file):
    """
    Parse a requirements file into a list of Requirements

    See: pip/req.py:parse_requirements() and
    https://pip.pypa.io/en/stable/reference/pip_install/#requirements-file-format

    :param req_str_or_file: a string or file like object containing requirements
    :returns: a *generator* of Requirement objects
    """
    filename = getattr(req_str_or_file, 'name', None)
    reqstr = req_str_or_file
    try:
        # Python 2.x compatibility
        if not isinstance(req_str_or_file, basestring):
            reqstr = req_str_or_file.read()
    except NameError:
        # Python 3.x only
        if not isinstance(req_str_or_file, str):
            reqstr = req_str_or_file.read()

    # To deal with lines broken via a backslash
    initial_zero_based_line_idx = 0
    accumulator_line_parts = []

    for zero_based_line_idx, original_line in enumerate(reqstr.splitlines()):

        if original_line.endswith('\\'):
            accumulator_line_parts.append(original_line.rstrip('\\'))
            continue

        # Construct a full line from pieces broken by backslashes
        accumulator_line_parts.append(original_line)
        original_line_idxs = (initial_zero_based_line_idx, zero_based_line_idx)
        initial_zero_based_line_idx = zero_based_line_idx + 1
        line = ' '.join(accumulator_line_parts)
        accumulator_line_parts = []

        line = line.strip()

        if line == '':
            continue
        elif not line or line.startswith('#'):
            # comments are lines that start with # only
            continue
        elif line.startswith('--trusted-host'):
            # unsupported
            continue
        elif line.startswith('-r') or line.startswith('--requirement'):
            new_filename = line.split()[1]
            new_file_path = os.path.join(os.path.dirname(filename or '.'),
                                         new_filename)
            with open(new_file_path) as f:
                for requirement in parse(f):
                    yield requirement
        elif line.startswith('-f') or line.startswith('--find-links') or \
                line.startswith('-i') or line.startswith('--index-url') or \
                line.startswith('--extra-index-url') or \
                line.startswith('--no-index'):
            warnings.warn('Private repos not supported. Skipping.')
            continue
        elif line.startswith('-Z') or line.startswith('--always-unzip'):
            warnings.warn('Unused option --always-unzip. Skipping.')
            continue
        elif line.startswith('-'):
            warnings.warn('Line starts with an option (%s). Skipping.' % \
                line.split()[0])
            continue
        else:
            req = Requirement.parse(line)
            req.provenance = (
                filename,
                original_line_idxs[0] + 1,
                original_line_idxs[1] + 1,
            )
            yield req
